import argparse
import jsons
import logging
import os
from overrides import overrides
from typing import List, Tuple

from sacrerouge.commands import RootSubcommand
from sacrerouge.common import Params
from sacrerouge.common.logging import prepare_global_logging
from sacrerouge.common.util import import_module_and_submodules
from sacrerouge.data import EvalInstance, Metrics, MetricsDict
from sacrerouge.data.dataset_readers import DatasetReader
from sacrerouge.io import JsonlWriter
from sacrerouge.metrics import Metric

logger = logging.getLogger(__name__)


def load_metrics(params: Params) -> List[Metric]:
    metrics = []
    for metric_params in params.pop('metrics'):
        metric = Metric.from_params(metric_params)
        metrics.append(metric)
    return metrics


def get_initial_micro_list(instances: List[EvalInstance]) -> List[Metrics]:
    micro_list = []
    for instance in instances:
        micro_list.append(Metrics(instance.instance_id, instance.summarizer_id, instance.summarizer_type))
    return micro_list


def evaluate_instances(instances: List[EvalInstance], metrics: List[Metric]) -> Tuple[MetricsDict, List[Metrics]]:
    macro = MetricsDict()
    micro_list = get_initial_micro_list(instances)

    for metric in metrics:
        # Prepare the input arguments
        summary_args = []
        for field in metric.required_summary_fields:
            summary_args.append([instance.fields[field].to_input() for instance in instances])

        context_args = []
        for field in metric.required_context_fields:
            context_args.append([instance.fields[field].to_input() for instance in instances])

        # Score all the summaries
        this_macro, this_micro_list = metric.evaluate(*summary_args, *context_args)

        # Update the global metrics dictionaries
        macro.update(this_macro)
        for micro, this_micro in zip(micro_list, this_micro_list):
            micro.metrics.update(this_micro)

    return macro, micro_list


def save_evaluation_results(macro_results: MetricsDict,
                            micro_results_list: List[Metrics],
                            macro_output_json: str,
                            micro_output_jsonl: str,
                            silent: bool) -> None:
    dirname = os.path.dirname(macro_output_json)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    serialized_macro = jsons.dumps({'metrics': macro_results}, jdkwargs={'indent': 2})
    with open(macro_output_json, 'w') as out:
        out.write(serialized_macro)
    if not silent:
        logger.info(serialized_macro)

    with JsonlWriter(micro_output_jsonl) as out:
        for metrics_dict in micro_results_list:
            out.write(metrics_dict)


def add_evaluate_arguments(parser: argparse.ArgumentParser, include_config_arguments: bool) -> None:
    if include_config_arguments:
        parser.add_argument(
            '--config',
            type=str,
            help='The config file that specifies the dataset reader and metrics',
            required=True
        )
        parser.add_argument(
            '--overrides',
            type=str,
            help='A serialized json that will override the parameters passed in "config"'
        )

    parser.add_argument(
        '--macro-output-json',
        type=str,
        help='The path to where the system-level metrics should be written',
        required=True
    )
    parser.add_argument(
        '--micro-output-jsonl',
        type=str,
        help='The path to where the input-level metrics should be written',
        required=True
    )
    parser.add_argument(
        '--log-file',
        type=str,
        help='The file where the log should be written'
    )
    parser.add_argument(
        '--silent',
        action='store_true',
        help='Controls whether the log should be written to stdout'
    )
    parser.add_argument(
        '--include-packages',
        nargs='+',
        help='A list of additional packages to include'
    )


@RootSubcommand.register('evaluate')
class EvaluateSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Evaluate a summarization model'
        self.parser = parser.add_parser('evaluate', description=description, help=description)
        add_evaluate_arguments(self.parser, True)
        self.parser.set_defaults(func=self.run)

    @overrides
    def run(self, args):
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        import_module_and_submodules('sacrerouge')
        include_packages = args.include_packages or []
        for package in include_packages:
            import_module_and_submodules(package)

        params = Params.from_file(args.config, args.overrides)
        dataset_reader = DatasetReader.from_params(params.pop('dataset_reader'))
        metrics = load_metrics(params)

        input_files = params.pop('input_files')
        if isinstance(input_files, str):
            input_files = [input_files]

        instances = dataset_reader.read(*input_files)
        macro, micro_list = evaluate_instances(instances, metrics)

        save_evaluation_results(macro, micro_list, args.macro_output_json, args.micro_output_jsonl, args.silent)



import spacy

from sacrerouge.common.testing import FIXTURES_ROOT
from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.io import JsonlReader
from sacrerouge.metrics import DecomposedRouge


class TestDecomposedRouge(ReferenceBasedMetricTestCase):
    def test_spacy_model_version(self):
        # These results rely on having spacy en_core_web_sm version 2.3.1 installed
        nlp = spacy.load('en_core_web_sm')
        assert nlp.meta['version'] == '2.3.1'

    def test_regression(self):
        # This is a regression test, not necessarily a test for correctness. The expected output
        # is really long, so we saved it in a file
        metric = DecomposedRouge()
        expected_output = JsonlReader(f'{FIXTURES_ROOT}/data/decomposed-rouge/expected-output.jsonl').read()
        super().assert_expected_output(metric, expected_output)

    def test_order_invariant(self):
        metric = DecomposedRouge()
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['decomposed-rouge'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'decomposed-rouge'])


import argparse
import logging
import os
from collections import defaultdict
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List, Dict, Tuple

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.data import MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric

logger = logging.getLogger(__name__)


@Metric.register('meteor')
class Meteor(ReferenceBasedMetric):
    def __init__(self, meteor_root: str = f'{DATA_ROOT}/metrics/METEOR'):
        super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())
        self.meteor_root = meteor_root
        if not os.path.exists(meteor_root):
            raise Exception(f'Path "{meteor_root}" does not exist. Have you setup METEOR?')

    def _flatten_summaries(self, summaries_list: List[List[SummaryType]]) -> List[List[str]]:
        flattened_list = []
        for summaries in summaries_list:
            flattened_list.append([])
            for summary in summaries:
                if isinstance(summary, list):
                    summary = ' '.join(summary)
                flattened_list[-1].append(summary)
        return flattened_list

    def _parse_meteor_stdout(self, stdout: str) -> Tuple[float, List[float]]:
        lines = stdout.splitlines()
        assert lines[11].startswith('Segment 1 score'), 'Unexpected Meteor stdout format'

        individual_scores = []
        for line in lines[11:]:
            if not line.startswith('Segment'):
                break
            columns = line.split()
            individual_scores.append(float(columns[3]))

        assert lines[-1].startswith('Final score:'), 'Unexpected Meteor stdout format'
        columns = lines[-1].split()
        final_score = float(columns[2])

        return final_score, individual_scores

    def _aggregate_summary_scores(self,
                                  summaries_list: List[List[str]],
                                  references_list: List[List[str]],
                                  tuple_to_indices: Dict[Tuple[int, int], List[int]],
                                  individual_scores: List[float]) -> List[List[MetricsDict]]:
        metrics_lists = []
        for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
            metrics_lists.append([])
            for j, summary in enumerate(summaries):
                scores = [individual_scores[index] for index in tuple_to_indices[(i, j)]]
                metrics_lists[-1].append(MetricsDict({
                    'METEOR': sum(scores) / len(scores)
                }))
        return metrics_lists

    def _run(self,
             summaries_list: List[List[SummaryType]],
             references_list: List[List[SummaryType]]) -> Tuple[MetricsDict, List[List[MetricsDict]]]:
        summaries_list = self._flatten_summaries(summaries_list)
        references_list = self._flatten_summaries(references_list)

        with TemporaryDirectory() as temp_dir:
            # As far as I can tell, the input only allows for one reference
            # per input, so we need to write an instance for every pair and then
            # aggregate the output
            summaries_file = f'{temp_dir}/summaries.txt'
            references_file = f'{temp_dir}/references.txt'
            index = 0
            tuple_to_indices = defaultdict(list)
            with open(summaries_file, 'w') as out_summaries:
                with open(references_file, 'w') as out_references:
                    for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
                        for j, summary in enumerate(summaries):
                            for reference in references:
                                out_summaries.write(summary + '\n')
                                out_references.write(reference + '\n')
                                tuple_to_indices[(i, j)].append(index)
                                index += 1

            # Run meteor
            command = [
                'java', '-jar', f'{self.meteor_root}/meteor-1.5/meteor-1.5.jar',
                summaries_file, references_file,
                '-l', 'en',
                '-norm'
            ]

            logger.info(f'Running METEOR command: "{command}"')
            process = Popen(command, stdout=PIPE, stderr=PIPE)
            stdout, _ = process.communicate()

            final_score, individual_scores = self._parse_meteor_stdout(stdout.decode())

            macro_metrics = MetricsDict({'METEOR': final_score})
            micro_metrics_list = self._aggregate_summary_scores(summaries_list, references_list, tuple_to_indices, individual_scores)
            return macro_metrics, micro_metrics_list

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        _, micro_metrics_lists = self._run(summaries_list, references_list)
        return micro_metrics_lists

    def evaluate(self,
                 summaries: List[SummaryType],
                 references_list: List[List[ReferenceType]]) -> Tuple[MetricsDict, List[MetricsDict]]:
        summaries_list = [[summary] for summary in summaries]
        macro_metrics, micro_metrics_lists = self._run(summaries_list, references_list)
        micro_metrics_list = [metrics_list[0] for metrics_list in micro_metrics_lists]
        return macro_metrics, micro_metrics_list


@MetricSetupSubcommand.register('meteor')
class MeteorSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the METEOR metric'
        self.parser = parser.add_parser('meteor', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        commands = [
            f'mkdir -p {DATA_ROOT}/metrics/METEOR',
            f'cd {DATA_ROOT}/metrics/METEOR',
            f'wget https://www.cs.cmu.edu/~alavie/METEOR/download/meteor-1.5.tar.gz',
            f'tar xzvf meteor-1.5.tar.gz'
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()
        if process.returncode == 0:
            print('METEOR setup success')
        else:
            print('METEOR setup failure')



from spacy.tokens import Token
from typing import Dict, List

from sacrerouge.metrics.decomposed_rouge.categorizers import Categorizer, TupleCategorizer


class DependencyCategorizer(Categorizer):
    def __init__(self, name: str, relations: List[str]):
        super().__init__(name)
        self.relations = set(relations)

    def is_match(self, token1: Token, token2: Token) -> bool:
        return token1.dep_ in self.relations and token2.dep_ in self.relations

    def is_candidate(self, token: Token) -> bool:
        return token.dep_ in self.relations


class DependencyVerbRelationsCategorizer(TupleCategorizer):
    def __init__(self, name: str, relations: List[str]) -> None:
        super().__init__(name)
        self.relations = set(relations)

    def _get_children(self, tokens: List[Token], head: Token) -> List[Token]:
        children = []
        for token in tokens:
            if token.head == head:
                children.append(token)
        return children

    def get_tuples(self, tokens: List[Token]) -> List[Dict[str, int]]:
        tuples = []
        for token in tokens:
            if token.pos_ == 'VERB':
                children = self._get_children(tokens, token)

                # Filter to only children with tags we care about
                children = list(filter(lambda child: child.dep_ in self.relations, children))

                # If the remaining children have the same set of relations that we're
                # looking for, we keep the match. Otherwise, I don't know what to do
                if len(self.relations) == len(set(child.dep_ for child in children)):
                    match = {'VERB': token._.index}
                    for child in children:
                        match[child.dep_] = child._.index
                    tuples.append(match)

        return tuples



import os
import pytest
import unittest
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader


_en_file_path = 'datasets/multiling/multiling2011/en.metrics.jsonl'
_fr_file_path = 'datasets/multiling/multiling2011/fr.metrics.jsonl'


class TestMultiLing2011Metrics(unittest.TestCase):
    def _convert_to_dicts(self, metrics_list: List[Metrics]) -> Dict[str, Dict[str, Metrics]]:
        metrics_dicts = defaultdict(dict)
        for metrics in metrics_list:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics.metrics
        return metrics_dicts

    @pytest.mark.skipif(not os.path.exists(_en_file_path), reason='MultiLing 2011 English data does not exist')
    def test_en_summary_level(self):
        # Test a few random metrics to ensure the data was parsed correctly
        metrics_list = JsonlReader(_en_file_path, Metrics).read()
        metrics_dicts = self._convert_to_dicts(metrics_list)

        assert metrics_dicts['M004']['1']['rouge-1']['precision'] == pytest.approx(50.525, abs=1e-3)
        assert metrics_dicts['M004']['1']['rouge-1']['recall'] == pytest.approx(50.459, abs=1e-3)
        assert metrics_dicts['M004']['1']['rouge-1']['f1'] == pytest.approx(50.492, abs=1e-3)

        assert metrics_dicts['M009']['2']['rouge-2']['precision'] == pytest.approx(17.647, abs=1e-3)
        assert metrics_dicts['M009']['2']['rouge-2']['recall'] == pytest.approx(18.072, abs=1e-3)
        assert metrics_dicts['M009']['2']['rouge-2']['f1'] == pytest.approx(17.857, abs=1e-3)

        assert metrics_dicts['M002']['5']['MeMoG'] == pytest.approx(0.32643265, abs=1e-4)
        assert metrics_dicts['M004']['3']['MeMoG'] == pytest.approx(0.57679360, abs=1e-4)

        assert metrics_dicts['M000']['1']['grade'] == [2, 3, 3]
        assert metrics_dicts['M006']['4']['grade'] == [1, 3, 1]

        assert metrics_dicts['M000']['3']['length_aware_grade'] == [2.88, 3.83, 3.83]
        assert metrics_dicts['M006']['4']['length_aware_grade'] == [1.00, 3.00, 1.00]

    @pytest.mark.skipif(not os.path.exists(_fr_file_path), reason='MultiLing 2011 French data does not exist')
    def test_fr_summary_level(self):
        # Test a few random metrics to ensure the data was parsed correctly
        metrics_list = JsonlReader(_fr_file_path, Metrics).read()
        metrics_dicts = self._convert_to_dicts(metrics_list)

        assert metrics_dicts['M004']['1']['rouge-1']['precision'] == pytest.approx(42.791, abs=1e-3)
        assert metrics_dicts['M004']['1']['rouge-1']['recall'] == pytest.approx(44.753, abs=1e-3)
        assert metrics_dicts['M004']['1']['rouge-1']['f1'] == pytest.approx(43.750, abs=1e-3)

        assert metrics_dicts['M009']['2']['rouge-2']['precision'] == pytest.approx(17.770, abs=1e-3)
        assert metrics_dicts['M009']['2']['rouge-2']['recall'] == pytest.approx(17.813, abs=1e-3)
        assert metrics_dicts['M009']['2']['rouge-2']['f1'] == pytest.approx(17.791, abs=1e-3)

        assert metrics_dicts['M002']['5']['MeMoG'] == pytest.approx(0.24885254, abs=1e-4)
        assert metrics_dicts['M004']['3']['MeMoG'] == pytest.approx(0.46445730, abs=1e-4)

        assert metrics_dicts['M000']['1']['grade'] == [3, 1, 3]
        assert metrics_dicts['M006']['4']['grade'] == [1, 3, 3]

        assert metrics_dicts['M000']['3']['length_aware_grade'] == [2.73, 2.73, 3.63]
        assert metrics_dicts['M006']['4']['length_aware_grade'] == [1.00, 3.00, 3.00]



import argparse
import lxml.html
import re
import tarfile
from collections import defaultdict
from io import BytesIO
from nltk.tokenize import sent_tokenize
from typing import Any, Dict, List

from sacrerouge.io import JsonlWriter


def load_documents(documents_tar_path: str):
    main_documents_ids = defaultdict(list)
    update_documents_ids = defaultdict(lambda: defaultdict(list))
    documents = {}
    selectors = {}
    with tarfile.open(documents_tar_path, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2007_Summarization_Documents/duc2007_testdocs.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                if member.isfile():
                    path = member.name.split('/')
                    filename = path[-1]
                    instance_id = path[2][:-1].lower()

                    selector = path[2][-1]
                    if instance_id not in selectors:
                        selectors[instance_id] = selector

                    if path[1] == 'main':
                        main_documents_ids[instance_id].append(filename)
                    else:
                        group = path[3][-1]
                        update_documents_ids[instance_id][group].append(filename)

                    date = filename[3:11]
                    year, month, day = date[:4], date[4:6], date[6:]
                    date = f'{year}-{month}-{day}'

                    html = inner_tar.extractfile(member).read().decode()
                    tree = lxml.html.document_fromstring(html)

                    headline_nodes = tree.xpath('//headline')
                    if headline_nodes:
                        headline = headline_nodes[0].text_content().strip()
                    else:
                        headline = None

                    text = tree.xpath('//text')[0].text_content().strip()
                    text = re.sub('\s+', ' ', text)
                    sentences = sent_tokenize(text)
                    document = {
                        'filename': filename,
                        'headline': headline,
                        'date': date,
                        'text': sentences
                    }

                    if filename in documents:
                        assert documents[filename] == document
                    else:
                        documents[filename] = document

    return main_documents_ids, update_documents_ids, documents, selectors


def load_main_summaries(main_eval_tar_path: str):
    summaries = defaultdict(list)
    with tarfile.open(main_eval_tar_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('mainEval/ROUGE/models/'):
                path = member.name.split('/')
                instance_id = path[-1].split('.')[0].lower()
                annotator = path[-1].split('.')[-1]
                sentences = tar.extractfile(member).read().decode(errors='replace').splitlines()
                summary = {
                    'annotator': annotator,
                    'text': sentences
                }
                summaries[instance_id].append(summary)
    return summaries


def load_update_summaries(update_eval_tar_path: str):
    summaries = defaultdict(lambda: defaultdict(list))
    with tarfile.open(update_eval_tar_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('updateEval/ROUGE/models/'):
                path = member.name.split('/')
                instance_id = path[-1].split('.')[0][:-2].lower()
                group = path[-1].split('.')[0][-1]
                annotator = path[-1].split('.')[-1]
                sentences = tar.extractfile(member).read().decode(errors='replace').splitlines()
                summary = {
                    'annotator': annotator,
                    'text': sentences
                }
                summaries[instance_id][group].append(summary)
    return summaries


def load_topics(topics_file_path: str):
    tree = lxml.html.parse(topics_file_path)
    topics = {}
    for node in tree.xpath('//topic'):
        instance_id = node.xpath('num')[0].text_content().strip()[:-1].lower()
        title = node.xpath('title')[0].text_content().strip()
        narr = sent_tokenize(node.xpath('narr')[0].text_content().strip())
        topics[instance_id] = {
            'title': title,
            'narrative': narr
        }
    return topics


def save_main_data(document_ids: Dict[str, List[str]],
                   documents: Dict[str, List[str]],
                   summaries: Dict[str, List[List[str]]],
                   topics: Dict[str, Dict[str, Any]],
                   selectors: Dict[str, str],
                   file_path: str):
    with JsonlWriter(file_path) as out:
        for instance_id in sorted(document_ids.keys()):
            cluster_docs = [documents[doc_id] for doc_id in document_ids[instance_id]]
            cluster_summaries = summaries[instance_id]
            cluster_topic = topics[instance_id]
            selector = selectors[instance_id]
            assert len(cluster_summaries) == 4
            data = {
                'instance_id': instance_id,
                'selector': selector,
                'topic': cluster_topic,
                'documents': cluster_docs,
                'summaries': cluster_summaries
            }
            out.write(data)


def save_update_data(clusters: Dict[str, Dict[str, List[str]]],
                     documents: Dict[str, List[str]],
                     summaries: Dict[str, Dict[str, List[List[str]]]],
                     topics: Dict[str, Dict[str, Any]],
                     selectors: Dict[str, str],
                     file_path_A: str,
                     file_path_B: str,
                     file_path_C: str,
                     file_path_A_B_C: str):
    with JsonlWriter(file_path_A) as out_A:
        with JsonlWriter(file_path_B) as out_B:
            with JsonlWriter(file_path_C) as out_C:
                with JsonlWriter(file_path_A_B_C) as out_A_B_C:
                    for cluster in sorted(clusters.keys()):
                        selector = selectors[cluster]

                        docs1 = [documents[doc_id] for doc_id in clusters[cluster]['A']]
                        docs2 = [documents[doc_id] for doc_id in clusters[cluster]['B']]
                        docs3 = [documents[doc_id] for doc_id in clusters[cluster]['C']]

                        summaries1 = summaries[cluster]['A']
                        summaries2 = summaries[cluster]['B']
                        summaries3 = summaries[cluster]['C']
                        assert len(summaries1) == 4
                        assert len(summaries2) == 4
                        assert len(summaries3) == 4

                        topic = topics[cluster]
                        data_A = {
                            'instance_id': cluster + '-A',
                            'selector': selector,
                            'topic': topic,
                            'documents': docs1,
                            'summaries': summaries1,
                        }
                        data_B = {
                            'instance_id': cluster + '-B',
                            'selector': selector,
                            'topic': topic,
                            'documents': docs2,
                            'summaries': summaries2,
                        }
                        data_C = {
                            'instance_id': cluster + '-C',
                            'selector': selector,
                            'topic': topic,
                            'documents': docs3,
                            'summaries': summaries3,
                        }
                        data_A_B_C = {
                            'instance_id': cluster,
                            'selector': selector,
                            'topic': topic,
                            'documents_A': docs1,
                            'documents_B': docs2,
                            'documents_C': docs3,
                            'summaries_A': summaries1,
                            'summaries_B': summaries2,
                            'summaries_C': summaries3
                        }
                        out_A.write(data_A)
                        out_B.write(data_B)
                        out_C.write(data_C)
                        out_A_B_C.write(data_A_B_C)


def setup(data_root: str, output_dir: str):
    documents_tar = f'{data_root}/from-nist/DUC2007_Summarization_Documents.tgz'
    main_eval_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/results/mainEval.tar.gz'
    update_eval_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/results/updateEval.tar.gz'
    main_topics = f'{data_root}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/testdata/duc2007_topics.sgml'
    update_topics = f'{data_root}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/testdata/duc2007_UPDATEtopics.sgml'
    main(documents_tar, main_eval_tar, update_eval_tar, main_topics, update_topics, output_dir)


def main(documents_tar, main_eval_tar, update_eval_tar, main_topics, update_topics, output_dir):
    main_documents_ids, update_documents_ids, documents, selectors \
        = load_documents(documents_tar)
    main_summaries = load_main_summaries(main_eval_tar)
    update_summaries = load_update_summaries(update_eval_tar)
    main_topics = load_topics(main_topics)
    update_topics = load_topics(update_topics)

    save_main_data(main_documents_ids, documents, main_summaries, main_topics, selectors, f'{output_dir}/task1.jsonl')
    save_update_data(update_documents_ids, documents, update_summaries, update_topics, selectors,
                     f'{output_dir}/task2.A.jsonl',
                     f'{output_dir}/task2.B.jsonl',
                     f'{output_dir}/task2.C.jsonl',
                     f'{output_dir}/task2.A-B-C.jsonl')


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('documents_tar')
    argp.add_argument('main_eval_tar')
    argp.add_argument('update_eval_tar')
    argp.add_argument('main_topics')
    argp.add_argument('update_topics')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.documents_tar, args.main_eval_tar, args.update_eval_tar,
         args.main_topics, args.update_topics, args.output_dir)



import argparse
import logging
from collections import defaultdict
from overrides import overrides
from typing import Dict, List

from sacrerouge.commands import RootSubcommand
from sacrerouge.common import Params
from sacrerouge.common.logging import prepare_global_logging
from sacrerouge.common.util import import_module_and_submodules
from sacrerouge.data import EvalInstance, Metrics
from sacrerouge.data.dataset_readers import DatasetReader
from sacrerouge.io import JsonlWriter
from sacrerouge.metrics import Metric

logger = logging.getLogger(__name__)


def add_score_arguments(parser: argparse.ArgumentParser, include_config_arguments: bool) -> None:
    if include_config_arguments:
        parser.add_argument(
            '--config',
            type=str,
            help='The config file that specifies the dataset reader and metrics',
            required=True
        )
        parser.add_argument(
            '--overrides',
            type=str,
            help='A serialized json that will override the parameters passed in "config"'
        )

    parser.add_argument(
        '--output-jsonl',
        type=str,
        help='The path to where the input-level metrics should be written',
        required=True
    )
    parser.add_argument(
        '--log-file',
        type=str,
        help='The file where the log should be written'
    )
    parser.add_argument(
        '--silent',
        action='store_true',
        help='Controls whether the log should be written to stdout'
    )
    parser.add_argument(
        '--include-packages',
        nargs='+',
        help='A list of additional packages to include'
    )
    parser.add_argument(
        '--disable-peer-jackknifing',
        action='store_true',
        help='Disable running jackknifing for peer summaries'
    )


def _load_metrics(params: Params) -> List[Metric]:
    metrics = []
    for metric_params in params.pop('metrics'):
        metric = Metric.from_params(metric_params)
        metrics.append(metric)
    return metrics


def _score_with_metric(metric: Metric,
                       instances: List[EvalInstance],
                       metrics_dicts: Dict[str, Dict[str, Metrics]],
                       disable_peer_jackknifing: bool = False) -> None:
    # The summaries need to be grouped based on identical context. For instance, we group all of the summaries
    # that have the same reference documents together. This can sometimes make calculating the metric faster. The
    # following variables assist doing this.
    #
    # Maintains a list of the unique contexts which group the summaries
    fields_list = []

    # A mapping from the context to its index in `fields_list`
    field_to_index = {}

    # A nested list that will be parallel to `fields_list`. The entry at index `i` is a list of instances which should
    # be scored with `fields_list[i]`
    instances_list = []

    # A nested list that will be parallel to `instances_list` which contains the summary-specific fields
    # for the corresponding instance
    summary_fields_lists = []

    # A nested list that will be parallel to `instances_list` which marks if the calculation for that (summary, context)
    # pair represents jackknifing or not
    jackknifing_flags = []

    for instance in instances:
        # Select just the relevant fields for this metric
        summary_fields = instance.fields.select_fields(metric.required_summary_fields)
        context_fields = instance.fields.select_fields(metric.required_context_fields)

        # Score the instance normally using all of the fields. However,
        # if the metric requires jackknifing and this is a reference summary,
        # the metric is comparable to the jackknifing metrics.
        is_jackknifing = metric.requires_jackknifing() and instance.summarizer_type == 'reference'

        if context_fields not in field_to_index:
            field_to_index[context_fields] = len(field_to_index)
            fields_list.append(context_fields)
            instances_list.append([])
            summary_fields_lists.append([])
            jackknifing_flags.append([])

        index = field_to_index[context_fields]
        instances_list[index].append(instance)
        summary_fields_lists[index].append(summary_fields)
        jackknifing_flags[index].append(is_jackknifing)

        # Potentially run jackknifing for the peers
        if not disable_peer_jackknifing and metric.requires_jackknifing() and instance.summarizer_type == 'peer':
            jk_fields_list = metric.jackknifer.get_jackknifing_fields_list(context_fields)
            if jk_fields_list:
                for jk_fields in jk_fields_list:
                    if jk_fields not in field_to_index:
                        field_to_index[jk_fields] = len(field_to_index)
                        fields_list.append(jk_fields)
                        instances_list.append([])
                        summary_fields_lists.append([])
                        jackknifing_flags.append([])

                    index = field_to_index[jk_fields]
                    instances_list[index].append(instance)
                    summary_fields_lists[index].append(summary_fields)
                    jackknifing_flags[index].append(True)

    # Construct the arguments that will be passed to the scoring method
    summary_args = []
    for name in metric.required_summary_fields:
        summary_args.append([[summary_fields[name].to_input() for summary_fields in summary_fields_list] for summary_fields_list in summary_fields_lists])

    context_args = []
    for name in metric.required_context_fields:
        context_args.append([fields[name].to_input() for fields in fields_list])

    # Score the summaries
    results_lists = metric.score_multi_all(*summary_args, *context_args)

    # Used to aggregate the jk results
    jk_results = defaultdict(lambda: defaultdict(list))

    for i, results_list in enumerate(results_lists):
        for j, results in enumerate(results_list):
            instance = instances_list[i][j]
            is_jackknifing = jackknifing_flags[i][j]
            if is_jackknifing:
                jk_results[instance.instance_id][instance.summarizer_id].append(results)
            else:
                metrics_dicts[instance.instance_id][instance.summarizer_id].metrics.update(results)

    # Aggregate the jk results
    for instance_id in jk_results.keys():
        for summarizer_id, results in jk_results[instance_id].items():
            result = sum(results) / len(results)
            for name, value in result.items():
                metrics_dicts[instance_id][summarizer_id].metrics[name + '_jk'] = value


def _get_initial_metrics_dicts(instances: List[EvalInstance]) -> Dict[str, Dict[str, Metrics]]:
    metrics_dicts = defaultdict(dict)
    for instance in instances:
        metrics = Metrics(instance.instance_id, instance.summarizer_id, instance.summarizer_type)
        metrics_dicts[instance.instance_id][instance.summarizer_id] = metrics
    return metrics_dicts


def score_instances(instances: List[EvalInstance],
                    metrics: List[Metric],
                    disable_peer_jackknifing: bool = False) -> Dict[str, Dict[str, Metrics]]:
    metrics_dicts = _get_initial_metrics_dicts(instances)
    for metric in metrics:
        _score_with_metric(metric, instances, metrics_dicts, disable_peer_jackknifing=disable_peer_jackknifing)
    return metrics_dicts


def save_score_results(metrics_dicts: Dict[str, Dict[str, Metrics]], output_file: str, silent: bool) -> None:
    with JsonlWriter(output_file) as out:
        for instance_id in sorted(metrics_dicts.keys()):
            for summarizer_id in sorted(metrics_dicts[instance_id].keys()):
                out.write(metrics_dicts[instance_id][summarizer_id])


@RootSubcommand.register('score')
class ScoreSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Score all of the inputs to evaluate a metric'
        self.parser = parser.add_parser('score', description=description, help=description)
        add_score_arguments(self.parser, True)
        self.parser.set_defaults(func=self.run)

    @overrides
    def run(self, args):
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        import_module_and_submodules('sacrerouge')
        include_packages = args.include_packages or []
        for package in include_packages:
            import_module_and_submodules(package)

        params = Params.from_file(args.config, args.overrides)
        dataset_reader = DatasetReader.from_params(params.pop('dataset_reader'))
        metrics = _load_metrics(params)

        input_files = params.pop('input_files')
        if isinstance(input_files, str):
            input_files = [input_files]

        instances = dataset_reader.read(*input_files)
        metrics_dicts = score_instances(instances, metrics, args.disable_peer_jackknifing)

        save_score_results(metrics_dicts, args.output_jsonl, args.silent)



from spacy.tokens import Token

from sacrerouge.metrics.decomposed_rouge.categorizers import Categorizer


class StopwordCategorizer(Categorizer):
    def __init__(self, data_dir: str):
        super().__init__('stopwords')
        self.stopwords = set(open(f'{data_dir}/smart_common_words.txt', 'r').read().splitlines())

    def is_match(self, token1: Token, token2: Token) -> bool:
        return token1.text.lower() in self.stopwords and token2.text.lower() in self.stopwords

    def is_candidate(self, token: Token) -> bool:
        return token.text.lower() in self.stopwords



import os
import zipfile
from collections import defaultdict
from io import BytesIO
from typing import Dict, List

from sacrerouge.io import JsonlWriter
from sacrerouge.datasets.multiling.util import LANGUAGE_CODES


def load_documents(data_path: str) -> Dict[str, Dict[str, List]]:
    documents = defaultdict(lambda: defaultdict(list))

    with zipfile.ZipFile(data_path, 'r') as zip:
        documents_zip_bytes = zip.read('PublishedCorpusFileSet/MultiLingPilot2011-SourceTextsV1.0.zip')
        with zipfile.ZipFile(BytesIO(documents_zip_bytes)) as documents_zip:
            for name in documents_zip.namelist():
                if name.startswith('SourceTexts/M'):
                    filename = os.path.basename(name)
                    key, language = filename.split('.')
                    language = language.lower()
                    instance_id = key[:4]

                    # The documents seem to use blank lines to separate paragraphs,
                    # but since we don't have a sentence tokenization tool for every
                    # language, we don't attempt to do any further preprocessing.
                    document_bytes = documents_zip.read(name)
                    text = BytesIO(document_bytes).read().decode(errors='replace')
                    lines = list(filter(None, map(lambda line: line.strip(), text.splitlines())))
                    document = {
                        'filename': filename,
                        'text': lines
                    }
                    documents[language][instance_id].append(document)
    return documents


def load_summaries(data_path: str) -> Dict[str, Dict[str, List]]:
    summaries = defaultdict(lambda: defaultdict(list))

    with zipfile.ZipFile(data_path, 'r') as zip:
        summaries_zip_bytes = zip.read('PublishedCorpusFileSet/MultiLingPilot2011-ModelSummaries.zip')
        with zipfile.ZipFile(BytesIO(summaries_zip_bytes)) as summaries_zip:
            for name in summaries_zip.namelist():
                if name.endswith('.250'):
                    path = name.split('/')
                    language = path[0].lower()
                    filename = path[1]
                    parts = filename.split('.')
                    instance_id = parts[0][:4]
                    annotator = parts[1]

                    # The summaries don't seem to have any structure, so
                    # just read in the full text.
                    lines = summaries_zip.read(name).decode().splitlines()
                    clean_lines = []
                    for line in lines:
                        line = line.strip()
                        if line:
                            clean_lines.append(line)
                    text = ' '.join(clean_lines)
                    summary = {
                        'annotator': annotator,
                        'text': text
                    }
                    summaries[language][instance_id].append(summary)
    return summaries


def save_data(documents: Dict[str, Dict[str, List]],
              summaries: Dict[str, Dict[str, List]],
              output_dir: str) -> None:
    for language in documents.keys():
        code = LANGUAGE_CODES[language]
        with JsonlWriter(f'{output_dir}/{code}.jsonl') as out:
            for instance_id in sorted(documents[language].keys()):
                these_documents = documents[language][instance_id]
                these_summaries = summaries[language][instance_id]
                out.write({
                    'instance_id': instance_id,
                    'documents': these_documents,
                    'summaries': these_summaries
                })


def setup(data_path: str, output_dir: str):
    documents = load_documents(data_path)
    summaries = load_summaries(data_path)
    save_data(documents, summaries, output_dir)



import os
import pytest
import unittest
from glob import glob

from sacrerouge.common.testing.util import load_metrics_dicts
from sacrerouge.io import JsonlReader

_mds_dir = 'datasets/multiling/multiling2013/mds'


@pytest.mark.skipif(not os.path.exists(_mds_dir), reason='MultiLing 2013 MDS data does not exist')
class TestMultiLing2013MDS(unittest.TestCase):
    def test_sanity_checks(self):
        file_paths = glob(f'{_mds_dir}/??.jsonl')
        assert len(file_paths) == 10  # 10 languages

        # Make sure each one has non-empty documents and summaries
        for file_path in file_paths:
            instances = JsonlReader(file_path).read()
            for instance in instances:
                assert len(instance['documents']) == 10
                for document in instance['documents']:
                    assert len(document['text']) > 0

                assert len(instance['summaries']) > 0
                for summary in instance['summaries']:
                    assert len(summary['text']) > 0

    def test_metrics(self):
        for language in ['ar', 'es', 'he', 'zh', 'cs', 'ro', 'el', 'en']:
            # Make sure summaries and metrics in parallel
            summaries = JsonlReader(f'{_mds_dir}/{language}.summaries.jsonl').read()
            metrics = JsonlReader(f'{_mds_dir}/{language}.metrics.jsonl').read()
            assert len(summaries) == len(metrics)

            for summary, metrics in zip(summaries, metrics):
                assert summary['instance_id'] == metrics['instance_id']
                assert summary['summarizer_id'] == metrics['summarizer_id']
                assert summary['summarizer_type'] == metrics['summarizer_type']

        # Test a few metrics
        metrics_dicts = load_metrics_dicts(f'{_mds_dir}/en.metrics.jsonl')
        assert metrics_dicts['M000']['ID1']['overall_responsiveness'] == [3.0, 3.0]
        assert metrics_dicts['M006']['ID61']['overall_responsiveness'] == [2.0, 2.0, 2.0]
        assert metrics_dicts['M100']['A']['overall_responsiveness'] == [4.0, 5.0, 5.0]

        metrics_dicts = load_metrics_dicts(f'{_mds_dir}/es.metrics.jsonl')
        assert metrics_dicts['M000']['A']['overall_responsiveness'] == [4.0, 3.0, 3.0]
        assert metrics_dicts['M006']['ID4']['overall_responsiveness'] == [5.0, 3.0, 4.0]
        assert metrics_dicts['M009']['ID2']['overall_responsiveness'] == [4.0, 5.0, 3.0]



import argparse
from overrides import overrides

from sacrerouge.common import Registrable
from sacrerouge.commands import RootSubcommand, MetricSetupSubcommand


@RootSubcommand.register('setup-metric')
class SetupMetricSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup an evaluation metric'
        self.parser = parser.add_parser('setup-metric', description=description, help=description)
        subparsers = self.parser.add_subparsers()

        # Add all of the metric setup commands using the registry
        for name, (cls_, _) in sorted(Registrable._registry[MetricSetupSubcommand].items()):
            cls_().add_subparser(subparsers)

        self.parser.set_defaults(func=self.run)

    @overrides
    def run(self, args):
        if 'subfunc' in dir(args):
            args.subfunc(args)
        else:
            self.parser.print_help()



import lxml.html
import re
import tarfile
from collections import defaultdict
from io import BytesIO
from nltk.tokenize import sent_tokenize
from typing import Dict, List, Tuple

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter


def load_see_file(tar, member):
    text = tar.extractfile(member).read().decode(errors='replace')
    tree = lxml.html.document_fromstring(text)
    nodes = list(tree.xpath('//a[@id]'))
    sentences = [node.text_content() for node in nodes]
    return sentences


def load_summaries(results_dir):
    sds_summaries = defaultdict(dict)
    mds_abs_summaries = defaultdict(lambda: defaultdict(dict))
    mds_ext_summaries = defaultdict(lambda: defaultdict(dict))

    for filename in ['baseline1peers.tar.gz', 'baseline2peers.tar.gz', 'baseline3peers.tar.gz', 'manualpeers.tar.gz', 'submittedpeers.tar.gz']:
        with tarfile.open(f'{results_dir}/abstracts/phase1/SEEpeers/{filename}', 'r') as tar:
            for member in tar.getmembers():
                if member.name.endswith('.html'):
                    path = member.name.split('/')
                    filename = path[-1]
                    parts = filename.split('.')

                    sentences = load_see_file(tar, member)
                    summarizer_id = parts[4]
                    if summarizer_id.isalpha():
                        summarizer_type = 'reference'
                    else:
                        summarizer_type = 'peer'
                    summary = {
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summarizer_type,
                        'text': sentences
                    }

                    if parts[1] == 'M':
                        instance_id = parts[0].lower()
                        length = parts[2]
                        if length == '010':
                            length = '10'
                        if length == '050':
                            length = '50'
                        mds_abs_summaries[instance_id][length][summarizer_id] = summary
                    else:
                        instance_id = parts[5]
                        sds_summaries[instance_id][summarizer_id] = summary

    for filename in ['model.extracts.tar', 'submissions.extracts.tar']:
        with tarfile.open(f'{results_dir}/extracts/{filename}', 'r') as tar:
            for member in tar.getmembers():
                # TODO
                pass

    return sds_summaries, mds_abs_summaries, mds_ext_summaries


def load_see_table(file_path: str, sds_metrics, mds_metrics):
    lines = open(file_path, 'r').read().splitlines()
    for line in lines[37:]:
        columns = line.split()
        assert len(columns) == 33

        summarizer_id = columns[8]
        is_multi = columns[1] == 'M'
        if is_multi:
            instance_id = columns[0].lower()
            length = columns[3]
            if length == '010':
                length = '10'
            if length == '050':
                length = '50'
            metrics = mds_metrics[instance_id][length][summarizer_id]
        else:
            instance_id = columns[2]
            metrics = sds_metrics[instance_id][summarizer_id]

        for i in range(12):
            if columns[10 + i] != '-':
                metrics['peer_quality'][f'Q{i + 1}'] = int(columns[10 + i])

        metrics['unmarked_units_related_to_subject'] = float(columns[22])
        metrics['num_peer_units'] = int(columns[23])
        metrics['num_marked_peer_units'] = int(columns[24])
        metrics['num_unmarked_peer_units'] = int(columns[25])
        metrics['num_model_units'] = int(columns[26])
        metrics['coverage']['mean'] = float(columns[27])
        metrics['coverage']['median'] = float(columns[28])
        metrics['coverage']['std'] = float(columns[29])
        metrics['length_adjusted_coverage']['mean'] = float(columns[30])
        metrics['length_adjusted_coverage']['median'] = float(columns[31])
        metrics['length_adjusted_coverage']['std'] = float(columns[32])


def load_extracts_table(file_path: str, mds_metrics):
    lines = open(file_path, 'r').read().splitlines()
    for line in lines[14:]:
        columns = line.split()
        assert len(columns) == 13, columns

        instance_id = columns[0].lower()
        length = f'{columns[2]}e'
        summarizer_id = columns[4]

        if 'sentence_precision' not in mds_metrics[instance_id][length][summarizer_id]:
            mds_metrics[instance_id][length][summarizer_id]['sentence_precision'] = []
        if 'sentence_recall' not in mds_metrics[instance_id][length][summarizer_id]:
            mds_metrics[instance_id][length][summarizer_id]['sentence_recall'] = []

        mds_metrics[instance_id][length][summarizer_id]['sentence_precision'].append(float(columns[11]))
        mds_metrics[instance_id][length][summarizer_id]['sentence_recall'].append(float(columns[12]))


def get_sds_references(summaries, instance_id, summarizer_id):
    summarizer_ids = list(summaries[instance_id].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        references.append(summaries[instance_id][reference_id])
    return references


def get_mds_references(summaries, instance_id, length, summarizer_id):
    summarizer_ids = list(summaries[instance_id][length].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        references.append(summaries[instance_id][length][reference_id])
    return references


def save_sds_metrics(summaries: Dict[str, Dict[str, List[str]]],
                     metrics: Dict[str, Dict[str, List[int]]],
                     task_name: str,
                     output_dir: str):
    with JsonlWriter(f'{output_dir}/{task_name}.summaries.jsonl') as out_summaries:
        with JsonlWriter(f'{output_dir}/{task_name}.metrics.jsonl') as out_metrics:
            for instance_id in sorted(summaries.keys()):
                for summarizer_id in summaries[instance_id].keys():
                    summary = summaries[instance_id][summarizer_id]
                    instance_metrics = metrics[instance_id][summarizer_id]
                    if len(instance_metrics) == 0:
                        continue

                    references = get_sds_references(summaries, instance_id, summarizer_id)

                    out_summaries.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'summary': summary,
                        'references': references
                    })
                    out_metrics.write(Metrics(instance_id, summarizer_id, summary['summarizer_type'], instance_metrics))


def save_mds_metrics(summaries: Dict[str, Dict[str, List[str]]],
                     metrics: Dict[str, Dict[str, List[int]]],
                     task_name: str,
                     output_dir: str):
    for length in ['10', '50', '100', '200']:
        with JsonlWriter(f'{output_dir}/{task_name}.{length}.summaries.jsonl') as out_summaries:
            with JsonlWriter(f'{output_dir}/{task_name}.{length}.metrics.jsonl') as out_metrics:
                for instance_id in sorted(summaries.keys()):
                    for summarizer_id in summaries[instance_id][length].keys():
                        summary = summaries[instance_id][length][summarizer_id]
                        instance_metrics = metrics[instance_id][length][summarizer_id]
                        if len(instance_metrics) == 0:
                            continue

                        references = get_mds_references(summaries, instance_id, length, summarizer_id)

                        out_summaries.write({
                            'instance_id': instance_id,
                            'summarizer_id': summarizer_id,
                            'summarizer_type': summary['summarizer_type'],
                            'summary': summary,
                            'references': references
                        })
                        out_metrics.write(Metrics(instance_id, summarizer_id, summary['summarizer_type'], instance_metrics))


def setup(data_root: str, output_dir: str) -> None:
    results_dir = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2002/results'
    main(results_dir, output_dir)


def main(results_dir, output_dir):
    sds_summaries, mds_abs_summaries, mds_ext_summaries = load_summaries(results_dir)

    sds_metrics = defaultdict(lambda: defaultdict(MetricsDict))
    mds_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(MetricsDict)))
    load_see_table(f'{results_dir}/abstracts/phase1/short.results.table', sds_metrics, mds_metrics)
    load_extracts_table(f'{results_dir}/extracts/extracts.results.table', mds_metrics)

    save_sds_metrics(sds_summaries, sds_metrics, 'task1', output_dir)
    save_mds_metrics(mds_abs_summaries, mds_metrics, 'task2', output_dir)


import argparse
import tarfile
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter


def load_summaries(results_tar: str):
    summaries = defaultdict(dict)
    with tarfile.open(results_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('results/ROUGE/models/') or member.name.startswith('results/ROUGE/peers/'):
                parts = member.name.split('/')[-1].split('.')
                instance_id = parts[0].lower()
                summarizer_id = parts[-1]
                if summarizer_id.isalpha():
                    summarizer_type = 'reference'
                else:
                    summarizer_type = 'peer'
                sentences = tar.extractfile(member).read().decode(errors='replace').splitlines()
                if len(sentences) == 0:
                    print(f'Instance {instance_id} and summarizer {summarizer_id} summary is empty. Skipping')
                    continue

                summary = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': summarizer_type,
                    'text': sentences
                }
                if summarizer_id in summaries[instance_id]:
                    assert summaries[instance_id][summarizer_id] == summary
                summaries[instance_id][summarizer_id] = summary
    return summaries


def load_rouge_jk_output(eval_tar: str, file_path: str, metrics: Dict[str, Dict[str, MetricsDict]]) -> None:
    jk_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list))))
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile(file_path).read().decode().splitlines()
        for line in lines:
            columns = line.split()
            if len(columns) == 7 and columns[2] == 'Eval':
                summarizer_id = columns[0]
                rouge_metric = columns[1].lower() + '_jk'
                filename = columns[3].split('.')
                instance_id = filename[0].lower()
                summarizer_id = filename[-1]

                recall = float(columns[4][2:]) * 100
                precision = float(columns[5][2:]) * 100
                f1 = float(columns[6][2:]) * 100
                jk_metrics[instance_id][summarizer_id][rouge_metric]['recall'].append(recall)
                jk_metrics[instance_id][summarizer_id][rouge_metric]['precision'].append(precision)
                jk_metrics[instance_id][summarizer_id][rouge_metric]['f1'].append(f1)

        for instance_id in jk_metrics.keys():
            for summarizer_id in jk_metrics[instance_id].keys():
                for rouge_metric in jk_metrics[instance_id][summarizer_id].keys():
                    recalls = jk_metrics[instance_id][summarizer_id][rouge_metric]['recall']
                    precisions = jk_metrics[instance_id][summarizer_id][rouge_metric]['precision']
                    f1s = jk_metrics[instance_id][summarizer_id][rouge_metric]['f1']
                    metrics[instance_id][summarizer_id][rouge_metric] = {
                        'recall': sum(recalls) / len(recalls),
                        'precision': sum(precisions) / len(precisions),
                        'f1': sum(f1s) / len(f1s)
                    }


def load_responsiveness_table(eval_tar: str, metrics: Dict[str, Dict[str, MetricsDict]]) -> None:
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile('results/responsiveness/responsiveness.table').read().decode().splitlines()
        for line in lines[6:]:
            columns = line.split()
            instance_id = columns[0].lower()
            summarizer_id = columns[3]
            score = int(columns[4])
            metrics[instance_id][summarizer_id]['responsiveness'] = score


def load_linguistic_quality_table(eval_tar: str, metrics: Dict[str, Dict[str, MetricsDict]]):
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile('results/linguistic_quality/linguistic_quality.table').read().decode().splitlines()
        for line in lines[7:]:
            columns = line.split()
            instance_id = columns[0].lower()
            summarizer_id = columns[3]
            question = columns[4]
            score = int(columns[5])
            metrics[instance_id][summarizer_id]['linguistic_quality'][f'Q{question}'] = score


def load_pyramid_table(pyramid_tar: str, metrics: Dict[str, Dict[str, MetricsDict]]):
    with tarfile.open(pyramid_tar, 'r') as tar:
        # The "processed_pans.txt" file has obvious corrected errors, whereas "unprocessed_pans.txt" does not
        lines = tar.extractfile('processed_pans.txt').read().decode().splitlines()
        for line in lines:
            columns = line.split()
            instance_id = 'd' + columns[0].lower()
            summarizer_id = columns[1]

            # This file contains scores for both references and peers. However, the
            # annotation files show that only a portion of the references were used
            # to create the pyramid, and the remaining references were evaluated against
            # those pyramids. The pyramid references do not appear in this file.
            # Therefore, the pyramid scores between references and peers here can
            # be directly compared, and they don't need to be marked as jackknifed
            #
            # Also, for some clusters there are two sets of annotations. We just pick
            # whichever one comes last. I don't think it matters significantly
            metrics[instance_id][summarizer_id]['pyramid_score'] = float(columns[2])
            metrics[instance_id][summarizer_id]['modified_pyramid_score'] = float(columns[3])
            metrics[instance_id][summarizer_id]['num_scus'] = int(columns[4])
            metrics[instance_id][summarizer_id]['num_repetitions'] = int(columns[5])


def get_references(summaries, instance_id, summarizer_id):
    summarizer_ids = list(summaries[instance_id].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        references.append(summaries[instance_id][reference_id])
    return references


def save_metrics(summaries: Dict[str, Dict[str, List[str]]],
                 metrics: Dict[str, Dict[str, List[int]]],
                 output_dir: str):
    with JsonlWriter(f'{output_dir}/task1.summaries.jsonl') as out_summaries:
        with JsonlWriter(f'{output_dir}/task1.metrics.jsonl') as out_metrics:
            for instance_id in sorted(summaries.keys()):
                for summarizer_id in summaries[instance_id].keys():
                    summary = summaries[instance_id][summarizer_id]
                    instance_metrics = metrics[instance_id][summarizer_id]
                    references = get_references(summaries, instance_id, summarizer_id)

                    out_summaries.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'summary': summary,
                        'references': references
                    })
                    out_metrics.write(Metrics(instance_id, summarizer_id, summary['summarizer_type'], instance_metrics))


def setup(data_root: str, output_dir: str) -> None:
    results_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2005/results/NIST/results.tar'
    pyramid_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2005/results/Pyramid/DUC2005.tar.gz'
    main(results_tar, pyramid_tar, output_dir)


def main(results_tar, pyramid_tar, output_dir):
    summaries = load_summaries(results_tar)

    metrics = defaultdict(lambda: defaultdict(MetricsDict))
    load_rouge_jk_output(results_tar, 'results/ROUGE/rougejk.m.out', metrics)
    load_responsiveness_table(results_tar, metrics)
    load_linguistic_quality_table(results_tar, metrics)
    load_pyramid_table(pyramid_tar, metrics)

    save_metrics(summaries, metrics, output_dir)


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('results_tar')
    argp.add_argument('pyramid_tar')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.results_tar, args.pyramid_tar, args.output_dir)



import gzip
import lxml.html
import re
import tarfile
from collections import defaultdict
from io import BytesIO
from nltk.tokenize import sent_tokenize
from tqdm import tqdm
from typing import Dict, List

from sacrerouge.io import JsonlWriter

_CATEGORIES = [
    'Accidents and Natural Disasters',
    'Attacks',
    'Health and Safety',
    'Endangered Resources',
    'Investigations and Trials'
]


def load_clusters_and_topics(file_path: str):
    clusters = defaultdict(lambda: defaultdict(list))
    topics = {}
    # There is something weird that lxml does not like about the first
    # 35 lines, so we remove them
    html = open(file_path, 'r').read().splitlines()
    html = ' '.join(html[36:])
    tree = lxml.html.document_fromstring(html)
    for topic_node in tree.xpath('//topic'):
        cluster = topic_node.get('id')[:-1].lower()
        category = _CATEGORIES[int(topic_node.get('category').strip()) - 1]
        title = topic_node.xpath('title')[0].text.strip()
        topics[cluster] = {
            'title': title,
            'category': category
        }

        for doc_node in topic_node.xpath('docseta/doc'):
            # Get the filename by stripping off the LDC ID
            file_id = doc_node.get('id')
            filename = file_id[:-11]
            clusters[cluster]['A'].append(filename)

        for doc_node in topic_node.xpath('docsetb/doc'):
            # Get the filename by stripping off the LDC ID
            file_id = doc_node.get('id')
            filename = file_id[:-11]
            clusters[cluster]['B'].append(filename)

    return clusters, topics


def parse_story(doc_node):
    headline = None
    headline_nodes = list(doc_node.xpath(f'./headline'))
    if len(headline_nodes) > 0:
        headline = headline_nodes[0].text_content().strip()

    text = []
    for p_node in doc_node.xpath('./text/p'):
        paragraph = p_node.text_content().strip()
        paragraph = re.sub('\s+', ' ', paragraph)
        paragraph = sent_tokenize(paragraph)
        text.append(paragraph)

    return headline, text


def parse_other(root):
    # Everything is combined into a single paragraph.
    text = root.xpath('//text')[0].text_content().strip()
    text = re.sub('\s+', ' ', text)
    sentences = [sent_tokenize(text)]
    return sentences


def load_documents(gigaword_root: str, clusters: Dict[str, List[str]]):
    document_set = set()
    for groups in clusters.values():
        for documents in groups.values():
            for document in documents:
                document_set.add(document)

    documents = {}
    for document_id in tqdm(document_set):
        tqdm.write(document_id)

        corpus = document_id[:7].lower()
        gzip_name = document_id[:14].lower()
        file_path = f'{gigaword_root}/data/{corpus}/{gzip_name}.gz'

        date = document_id[8:16]
        year, month, day = date[:4], date[4:6], date[6:]
        date = f'{year}-{month}-{day}'

        html = gzip.open(file_path, 'rb').read().decode()
        tree = lxml.html.document_fromstring(html)

        doc_node = tree.xpath(f"body/doc[@id='{document_id}']")[0]

        # The documents come from English Gigaword. Different document
        # types have different structures, some of which we can parse
        # ("story"), but others we cannot ("other")
        document_type = tree.xpath('body/doc')[0].get('type')
        if document_type == 'story':
            headline, text = parse_story(doc_node)
        else:
            headline = None
            text = parse_other(doc_node)

        document = {
            'filename': document_id,
            'document_type': document_type,
            'headline': headline,
            'date': date,
            'text': text
        }
        documents[document_id] = document

    return documents


def load_summaries(eval_tar: str):
    summaries = defaultdict(lambda: defaultdict(list))
    with tarfile.open(eval_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('GuidedSumm2011_eval/ROUGE/models/'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[0].split('-')[0].lower()
                group = parts[0].split('-')[1]
                annotator = parts[4]

                sentences = tar.extractfile(member).read().decode(errors='replace').strip().splitlines()
                sentences = list(map(lambda sent: sent.strip(), sentences))
                summary = {
                    'annotator': annotator,
                    'text': sentences
                }
                summaries[instance_id][group].append(summary)
    return summaries


def save_instances(clusters, documents, summaries, topics, output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.A-B.jsonl') as out_A_B:
        with JsonlWriter(f'{output_dir}/task1.A.jsonl') as out_A:
            with JsonlWriter(f'{output_dir}/task1.B.jsonl') as out_B:
                for instance_id in clusters.keys():
                    topic = topics[instance_id]

                    documents_A = [documents[file_id] for file_id in clusters[instance_id]['A']]
                    documents_B = [documents[file_id] for file_id in clusters[instance_id]['B']]

                    summaries_A = summaries[instance_id]['A']
                    summaries_B = summaries[instance_id]['B']

                    out_A.write({
                        'instance_id': f'{instance_id}-A',
                        'topic': topic,
                        'documents': documents_A,
                        'summaries': summaries_A
                    })
                    out_B.write({
                        'instance_id': f'{instance_id}-B',
                        'topic': topic,
                        'documents': documents_B,
                        'summaries': summaries_B
                    })
                    out_A_B.write({
                        'instance_id': instance_id,
                        'topic': topic,
                        'documents_A': documents_A,
                        'documents_B': documents_B,
                        'summaries_A': summaries_A,
                        'summaries_B': summaries_B
                    })


def setup(gigaword_root: str, data_root: str, output_dir: str):
    eval_tar = f'{data_root}/scrapes/tac.nist.gov/protected/past/2011/GuidedSumm2011_eval.tgz'
    topics_file = f'{data_root}/scrapes/tac.nist.gov/protected/past/2011/GuidedSumm11_test_topics.xml.txt'
    main(gigaword_root, eval_tar, topics_file, output_dir)


def main(gigaword_root, eval_tar, topics_file, output_dir):
    clusters, topics = load_clusters_and_topics(topics_file)
    documents = load_documents(gigaword_root, clusters)
    summaries = load_summaries(eval_tar)
    save_instances(clusters, documents, summaries, topics, output_dir)


import unittest
from typing import List

from sacrerouge.common.testing import MULTILING_DOCUMENTS, MULTILING_SUMMARIES
from sacrerouge.data import MetricsDict
from sacrerouge.io import JsonlReader
from sacrerouge.metrics import Metric


class MetricTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.instance_ids = []
        cls.summaries = []
        cls.references_list = []
        with JsonlReader(MULTILING_SUMMARIES) as f:
            for instance in f:
                cls.instance_ids.append(instance['instance_id'])
                cls.summaries.append(instance['summary']['text'])
                references = []
                for reference in instance['references']:
                    references.append(reference['text'])
                cls.references_list.append(references)

        # Load the documents, grouped by instance id, then put them into a list
        # parallel with the instances
        cls.documents_dict = {}
        with JsonlReader(MULTILING_DOCUMENTS) as f:
            for instance in f:
                cls.documents_dict[instance['instance_id']] = [document['text'] for document in instance['documents']]
        cls.documents_list = []
        for instance_id in cls.instance_ids:
            cls.documents_list.append(cls.documents_dict[instance_id])

    def _assert_expected_output(self, metric: Metric, expected_output: List[MetricsDict], *args):
        """Ensures that the output from `score_all` is equal to the `expected_output`."""
        assert len(self.summaries) == len(expected_output)
        actual_output = metric.score_all(self.summaries, *args)
        assert len(actual_output) == len(expected_output)
        for i, (expected, actual) in enumerate(zip(expected_output, actual_output)):
            assert actual.approx_equal(MetricsDict(expected), abs=1e-4), f'Instance {i} not equal. Expected {expected}, actual {actual}'

    def _assert_order_invariant(self, metric: Metric, *args):
        """Ensures that the output from `score_multi_all` returns the same results, no matter the order."""
        # Reverse the summaries to create a new fake set of summaries that will be grouped together
        # with the same references. It does not matter that they're not the right references since we are
        # only testing to make sure the output doesn't change.
        faked = list(reversed(self.summaries))

        summaries_list = list(zip(*[self.summaries, faked]))
        metrics_lists1 = metric.score_multi_all(summaries_list, *args)
        metrics_lists1 = list(zip(*metrics_lists1))

        summaries_list = list(zip(*[faked, self.summaries]))
        metrics_lists2 = metric.score_multi_all(summaries_list, *args)
        metrics_lists2 = list(zip(*metrics_lists2))

        metrics_lists2 = list(reversed(metrics_lists2))
        for metrics_list1, metrics_list2 in zip(metrics_lists1, metrics_lists2):
            for metrics1, metrics2 in zip(metrics_list1, metrics_list2):
                assert metrics1.approx_equal(metrics2)


class DocumentBasedMetricTestCase(MetricTestCase):
    def assert_expected_output(self, metric: Metric, expected_output: List[MetricsDict]):
        self._assert_expected_output(metric, expected_output, self.documents_list)

    def assert_order_invariant(self, metric: Metric):
        self._assert_order_invariant(metric, self.documents_list)


class ReferenceBasedMetricTestCase(MetricTestCase):
    def assert_expected_output(self, metric: Metric, expected_output: List[MetricsDict]):
        self._assert_expected_output(metric, expected_output, self.references_list)

    def assert_order_invariant(self, metric: Metric):
        self._assert_order_invariant(metric, self.references_list)


class ReferencelessMetricTestCase(MetricTestCase):
    def assert_expected_output(self, metric: Metric, expected_output: List[MetricsDict]):
        self._assert_expected_output(metric, expected_output)

    def assert_order_invariant(self, metric: Metric):
        self._assert_order_invariant(metric)


import pytest

from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.data import MetricsDict
from sacrerouge.metrics import PythonRouge, Rouge
from sacrerouge.metrics.python_rouge import shorten_summary


class TestPythonRouge(ReferenceBasedMetricTestCase):
    def test_normalize_and_tokenize(self):
        """
        Tests to ensure the python version of Rouge correctly implements the
        perl implementation. The expected tokens were generated by editing the
        perl script to print the tokens during processing. The hyphens were added
        to the original because the perl implementation uses them to separate words.
        """
        rouge = PythonRouge(use_porter_stemmer=False, remove_stopwords=False)
        original = 'Xu Wenli, Wang Youchai, and Qin Yongmin, leading dissidents and '\
            'prominent members of the China-Democracy-Party, were found guilty of subversion ' \
            'and sentenced to 13, 11, and 12 years in prison, respectively.'
        expected = 'xu wenli wang youchai and qin yongmin leading dissidents and ' \
            'prominent members of the china democracy party were found guilty of subversion ' \
            'and sentenced to 13 11 and 12 years in prison respectively'.split()
        actual = rouge.normalize_and_tokenize_sentence(original)
        assert expected == actual

        rouge = PythonRouge(use_porter_stemmer=True, remove_stopwords=False)
        expected = 'xu wenli wang youchai and qin yongmin lead dissid and promin '\
            'member of the china democraci parti be find guilti of subvers and sentenc '\
            'to 13 11 and 12 year in prison respect'.split()
        actual = rouge.normalize_and_tokenize_sentence(original)
        assert expected == actual

        rouge = PythonRouge(use_porter_stemmer=False, remove_stopwords=True)
        expected = 'xu wenli wang youchai qin yongmin leading dissidents prominent '\
            'members china democracy party found guilty subversion sentenced 13 11 '\
            '12 years prison'.split()
        actual = rouge.normalize_and_tokenize_sentence(original)
        assert expected == actual

        rouge = PythonRouge(use_porter_stemmer=True, remove_stopwords=True)
        expected = 'xu wenli wang youchai qin yongmin lead dissid promin member china '\
            'democraci parti find guilti subvers sentenc 13 11 12 year prison'.split()
        actual = rouge.normalize_and_tokenize_sentence(original)
        assert expected == actual

    def test_shorten_summary(self):
        original = [
            'This is an example sentence, which is first.',
            'This is the second one.'
            'Finally, the third.'
        ]

        expected = [
            'This is an example'
        ]
        actual = shorten_summary(original, max_words=4)
        assert expected == actual

        expected = [
            'This is an example sentence, which is first.',
        ]
        actual = shorten_summary(original, max_words=8)
        assert expected == actual

        expected = [
            'This is an example sentence, which is first.',
            'This is'
        ]
        actual = shorten_summary(original, max_words=10)
        assert expected == actual

        actual = shorten_summary(original, max_words=16)
        assert original == actual
        actual = shorten_summary(original, max_words=100)
        assert original == actual

        expected = [
            'This is an'
        ]
        actual = shorten_summary(original, max_bytes=10)
        actual = shorten_summary(original, max_bytes=11)

        expected = [
            'This is an example sentence, which is first.',
        ]
        actual = shorten_summary(original, max_bytes=44)
        assert expected == actual

        expected = [
            'This is an example sentence, which is first.',
            'T'
        ]
        actual = shorten_summary(original, max_bytes=45)
        assert expected == actual

        actual = shorten_summary(original, max_bytes=87)
        assert original == actual
        actual = shorten_summary(original, max_bytes=88)
        assert original == actual

    def test_preprocessing(self):
        rouge = PythonRouge(max_words=5, remove_stopwords=True)
        original = [
            'Xu Wenli, Wang Youchai, and Qin Yongmin, leading dissidents and'
        ]
        expected = ['xu wenli wang youchai'.split()]
        actual = rouge.preprocess_summary(original)
        assert expected == actual

    def assert_same_as_rouge(self, python: MetricsDict, perl: MetricsDict):
        assert perl['rouge-1']['precision'] == pytest.approx(python['python-rouge-1']['precision'], abs=1e-2)
        assert perl['rouge-1']['recall'] == pytest.approx( python['python-rouge-1']['recall'], abs=1e-2)
        assert perl['rouge-1']['f1'] == pytest.approx( python['python-rouge-1']['f1'], abs=1e-2)
        assert perl['rouge-2']['precision'] == pytest.approx( python['python-rouge-2']['precision'], abs=1e-2)
        assert perl['rouge-2']['recall'] == pytest.approx( python['python-rouge-2']['recall'], abs=1e-2)
        assert perl['rouge-2']['f1'] == pytest.approx( python['python-rouge-2']['f1'], abs=1e-2)
        assert perl['rouge-l']['precision'] == pytest.approx( python['python-rouge-l']['precision'], abs=1e-2)
        assert perl['rouge-l']['recall'] == pytest.approx( python['python-rouge-l']['recall'], abs=1e-2)
        assert perl['rouge-l']['f1'] == pytest.approx( python['python-rouge-l']['f1'], abs=1e-2)

    def test_python_rouge_correctness(self):
        summary = [
            "His tenacity holds despite the summary trials and harsh punishments for Xu, Wang Youcai and Qin Yongmin prominent party principals from the provinces who were sentenced to 11 and 12 years and despite threatening signs from the ruling Communist Party.",
            "The dissidents Xu Wenli, who was sentenced Monday to 13 years in prison, Wang Youcai, who received an 11-year sentence, and Qin Yongming, who was reported to have received 12 years were charged with subversion.",
            "As police moved against Xu's friends, labor rights campaigner Liu Nianchun was taken from a prison camp outside Beijing and, with his wife and daughter, was put on a plane to Canada and then New York, his first taste of freedom in more than 3 1/2 years."
        ]
        gold_summaries = [
            [
                "While China plans to sign the International Covenant on Civil and Political Rights at the U.N., it is still harassing and arresting human rights campaigners.",
                "Three prominent leaders of the China Democratic Party were put to trial and sentenced to 11-, 12- and 13-year prison terms.",
                "Germany and the U.S. condemned the arrests.",
                "A labor rights activist was released and exiled to the U.S. to blunt any opposition to Communist rule.",
                "U.S. policy to encourage trade and diplomacy in hope of democratic reforms evidences failure, but the U.S. is continuing its policy of encouragement.",
                "Friends of jailed dissidents state that they will continue to campaign for change."
            ],
            [
                "The US trade-driven policy of expanded ties encouraging Chinese democracy is questioned.",
                "China signed rights treaties and dissidents used new laws to set up China Democracy Party, but China violates the new laws by persecuting dissidents.",
                "It regularly frees activists from prison then exiles them so they lose local influence.",
                "It arrested an activist trying to register a rights monitoring group.",
                "CP leader Jiang's hard-line speech and publicity for activists sentenced to long prison terms signals a renewed Chinese crackdown.",
                "A rights activist expected to be sacrificed in the cause of democracy.",
                "Germany called China's sentencing of dissidents unacceptable."
            ],
            [
                "After 2 years of wooing the West by signing international accords, apparently relaxing controls on free speech, and releasing and exiling three dissenters, China cracked down against political dissent in Dec 1998.",
                "Leaders of the China Democracy Party (CDP) were arrested and three were sentenced to jail terms of 11 to 13 years.",
                "The West, including the US, UK and Germany, reacted strongly.",
                "Clinton's China policy of engagement was questioned.",
                "China's Jiang Zemin stated economic reform is not a prelude to democracy and vowed to crush any challenges to the Communist Party or \"social stability\".",
                "The CDP vowed to keep working, as more leaders awaited arrest."
            ],
            [
                "Xu Wenli, Wang Youchai, and Qin Yongmin, leading dissidents and prominent members of the China Democracy Party, were found guilty of subversion and sentenced to 13, 11, and 12 years in prison, respectively.",
                "Soon after the sentencing, China's president, Jiang Zemin, delivered speeches in which he asserted that Western political system must not be adopted and vowed to crush challenges to Communist Party rule.",
                "The harsh sentences and speeches signal a crackdown on dissent, but Zha Jianguo, another Democracy Party leader, says he will continue to push for change.",
                "Western nations condemned the sentences as violations of U.N. rights treaties signed by China."
            ]
        ]

        compute_rouge_l = True
        use_porter_stemmer = False
        remove_stopwords = False
        rouge = Rouge(max_ngram=2,
                      compute_rouge_l=compute_rouge_l,
                      use_porter_stemmer=use_porter_stemmer,
                      remove_stopwords=remove_stopwords)
        python_rouge = PythonRouge(compute_rouge_l=compute_rouge_l,
                                   use_porter_stemmer=use_porter_stemmer,
                                   remove_stopwords=remove_stopwords)
        expected_metrics = rouge.score(summary, gold_summaries)
        actual_metrics = python_rouge.score(summary, gold_summaries)
        self.assert_same_as_rouge(actual_metrics, expected_metrics)

        use_porter_stemmer = False
        remove_stopwords = True
        rouge = Rouge(max_ngram=2,
                      compute_rouge_l=compute_rouge_l,
                      use_porter_stemmer=use_porter_stemmer,
                      remove_stopwords=remove_stopwords)
        python_rouge = PythonRouge(compute_rouge_l=compute_rouge_l,
                                   use_porter_stemmer=use_porter_stemmer,
                                   remove_stopwords=remove_stopwords)
        expected_metrics = rouge.score(summary, gold_summaries)
        actual_metrics = python_rouge.score(summary, gold_summaries)
        self.assert_same_as_rouge(actual_metrics, expected_metrics)

        use_porter_stemmer = True
        remove_stopwords = False
        rouge = Rouge(max_ngram=2,
                      compute_rouge_l=compute_rouge_l,
                      use_porter_stemmer=use_porter_stemmer,
                      remove_stopwords=remove_stopwords)
        python_rouge = PythonRouge(compute_rouge_l=compute_rouge_l,
                                   use_porter_stemmer=use_porter_stemmer,
                                   remove_stopwords=remove_stopwords)
        expected_metrics = rouge.score(summary, gold_summaries)
        actual_metrics = python_rouge.score(summary, gold_summaries)
        self.assert_same_as_rouge(actual_metrics, expected_metrics)

        use_porter_stemmer = True
        remove_stopwords = True
        rouge = Rouge(max_ngram=2,
                      compute_rouge_l=compute_rouge_l,
                      use_porter_stemmer=use_porter_stemmer,
                      remove_stopwords=remove_stopwords)
        python_rouge = PythonRouge(compute_rouge_l=compute_rouge_l,
                                   use_porter_stemmer=use_porter_stemmer,
                                   remove_stopwords=remove_stopwords)
        expected_metrics = rouge.score(summary, gold_summaries)
        actual_metrics = python_rouge.score(summary, gold_summaries)
        self.assert_same_as_rouge(actual_metrics, expected_metrics)

    def test_python_rouge_multiling(self):
        use_porter_stemmer = True
        remove_stopwords = False
        compute_rouge_l = True
        max_words = 100

        rouge = Rouge(max_ngram=2,
                      use_porter_stemmer=use_porter_stemmer,
                      remove_stopwords=remove_stopwords,
                      max_words=max_words,
                      compute_rouge_l=compute_rouge_l)
        python_rouge = PythonRouge(use_porter_stemmer=use_porter_stemmer,
                                   remove_stopwords=remove_stopwords,
                                   max_words=max_words,
                                   compute_rouge_l=compute_rouge_l)
        expected_metrics, _ = rouge.evaluate(self.summaries, self.references_list)
        actual_metrics, _ = python_rouge.evaluate(self.summaries, self.references_list)
        self.assert_same_as_rouge(actual_metrics, expected_metrics)

    def test_python_rouge_order_invariant(self):
        metric = PythonRouge()
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['python-rouge'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'python-rouge'])


import argparse
import os
import re
from collections import Counter
from nltk.stem import PorterStemmer
from overrides import overrides
from typing import Dict, List, Optional, Set, Tuple

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT
from sacrerouge.data import MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric


def shorten_summary(summary: SummaryType,
                    max_sentences: Optional[int] = None,
                    max_words: Optional[int] = None,
                    max_bytes: Optional[int] = None) -> List[str]:
    args = [max_sentences, max_words, max_bytes]
    if sum(1 if arg is not None else 0 for arg in args) not in [0, 1]:
        raise Exception(f'Only one of `max_sentences`, `max_words`, and `max_bytes` can be set.')

    # Maybe wrap the summary into a sentence
    if isinstance(summary, str):
        summary = [summary]

    shortened_summary = []
    if max_sentences is not None:
        shortened_summary = summary[:max_sentences]
    elif max_words is not None:
        budget = max_words
        for sentence in summary:
            tokens = sentence.split()[:budget]
            shortened_summary.append(' '.join(tokens))
            budget -= len(tokens)
            if budget <= 0:
                break
    elif max_bytes is not None:
        budget = max_bytes
        for sentence in summary:
            sentence = sentence[:budget].strip()
            shortened_summary.append(sentence)
            budget -= len(sentence)
            if budget <= 0:
                break
    else:
        shortened_summary = summary

    return shortened_summary


@Metric.register('python-rouge')
class PythonRouge(ReferenceBasedMetric):
    _non_alphanumeric_regex = re.compile('[^A-Za-z0-9]')

    def __init__(self,
                 ngram_orders: List[int] = [1, 2],
                 max_sentences: Optional[int] = None,
                 max_words: Optional[int] = None,
                 max_bytes: Optional[int] = None,
                 use_porter_stemmer: bool = True,
                 remove_stopwords: bool = False,
                 compute_rouge_l: bool = False,
                 rouge_data_dir: str = f'{DATA_ROOT}/metrics/ROUGE-1.5.5/data'):
        super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())
        self.ngram_orders = ngram_orders
        self.max_sentences = max_sentences
        self.max_words = max_words
        self.max_bytes = max_bytes
        self.use_porter_stemmer = use_porter_stemmer
        self.remove_stopwords = remove_stopwords
        self.compute_rouge_l = compute_rouge_l

        if not os.path.exists(rouge_data_dir):
            raise Exception(f'Path "{rouge_data_dir}" does not exist. PythonRouge requires data files from ROUGE. '
                            f'Have you setup ROUGE?')

        self.stemmer = PorterStemmer(PorterStemmer.ORIGINAL_ALGORITHM)
        self.stemmer_exceptions = self._load_stemmer_exceptions(rouge_data_dir)
        self.stopwords = self._load_stopwords(rouge_data_dir)

    def _load_stemmer_exceptions(self, root: str) -> Dict[str, str]:
        exceptions = {}
        for filename in ['adj.exc', 'adv.exc', 'noun.exc', 'verb.exc']:
            file_path = os.path.join(root, 'WordNet-2.0-Exceptions', filename)
            with open(file_path, 'r') as f:
                for line in f:
                    # I think there is a bug in the original perl script
                    # to construct the exceptions database. Some of the lines
                    # have more than 2 words on them, but the script only
                    # maps the first to the second, ignoring the third.
                    columns = line.strip().split()
                    exceptions[columns[0]] = columns[1]
        return exceptions

    def _load_stopwords(self, root: str) -> Set[str]:
        file_path = os.path.join(root, 'smart_common_words.txt')
        return set(open(file_path, 'r').read().splitlines())

    def normalize_and_tokenize_sentence(self, sentence: str) -> List[str]:
        sentence = PythonRouge._non_alphanumeric_regex.sub(' ', sentence)
        sentence = sentence.lower()
        tokens = []
        for token in sentence.split():
            if self.remove_stopwords and token in self.stopwords:
                continue
            if self.use_porter_stemmer and len(token) > 3:
                if token in self.stemmer_exceptions:
                    tokens.append(self.stemmer_exceptions[token])
                else:
                    tokens.append(self.stemmer.stem(token))
            else:
                tokens.append(token)
        return tokens

    def _normalize_and_tokenize_summary(self, summary: List[str]) -> List[str]:
        return [self.normalize_and_tokenize_sentence(sentence) for sentence in summary]

    def preprocess_summary(self, summary: SummaryType) -> List[List[str]]:
        summary = shorten_summary(summary, self.max_sentences, self.max_words, self.max_bytes)
        summary = self._normalize_and_tokenize_summary(summary)
        return summary

    def _count_ngrams(self, summary: SummaryType, n: int) -> Counter:
        counts = Counter()
        if isinstance(summary, str):
            summary = [summary]
        tokens = [token for sentence in summary for token in sentence]
        for i in range(len(tokens) - n + 1):
            ngram = ' '.join(tokens[i:i + n])
            counts[ngram] += 1
        return counts

    def _calculate_intersection(self, reference_counts: Counter, summary_counts: Counter) -> Tuple[float, float, float]:
        reference_total = sum(reference_counts.values())
        summary_total = sum(summary_counts.values())
        intersection = 0
        for ngram in summary_counts:
            intersection += min(summary_counts[ngram], reference_counts[ngram])
        return reference_total, summary_total, intersection

    def _calculate_pr_f1(self, reference_total: int, summary_total: int, intersection: int) -> Tuple[float, float, float]:
        precision = 0.0
        if summary_total != 0.0:
            precision = intersection / summary_total * 100
        recall = 0.0
        if reference_total != 0.0:
            recall = intersection / reference_total * 100
        if precision + recall == 0:
            f1 = 0.0
        else:
            f1 = 2 * (precision * recall) / (precision + recall)
        return precision, recall, f1

    def _longest_common_substring(self,
                                  tokens1: List[str],
                                  tokens2: List[str],
                                  hit_mask: List[int]) -> int:
        m, n = len(tokens1), len(tokens2)
        counter = [[0] * (n + 1) for x in range(m + 1)]
        pointers = [[None] * (n + 1) for x in range(m + 1)]
        for i in range(1, m + 1):
            for j in range(1, n + 1):
                if tokens1[i - 1] == tokens2[j - 1]:
                    counter[i][j] = counter[i - 1][j - 1] + 1
                    pointers[i][j] = '\\'
                elif counter[i - 1][j] >= counter[i][j - 1]:
                    counter[i][j] = counter[i - 1][j]
                    pointers[i][j] = '^'
                else:
                    counter[i][j] = counter[i][j - 1]
                    pointers[i][j] = '<'

        # Mark the hit_mask
        i, j = m, n
        while i != 0 and j != 0:
            if pointers[i][j] == '\\':
                i -= 1
                j -= 1
                hit_mask[i] = 1
            elif pointers[i][j] == '^':
                i -= 1
            elif pointers[i][j] == '<':
                j -= 1
            else:
                raise Exception(f'Unknown pointer: {pointers[i][j]}')

    def _calculate_rouge_l(self,
                           references: List[SummaryType],
                           summary: SummaryType):
        model_unigrams = self._count_ngrams(summary, 1)
        num_model_unigrams = sum(count for count in model_unigrams.values())

        if isinstance(summary, str):
            summary = [summary]
        references = [[reference] if isinstance(reference, str) else reference for reference in references]

        total_hit = 0
        total_base = 0
        for reference in references:
            temp_model_unigrams = Counter(model_unigrams)
            gold_unigrams = self._count_ngrams(reference, 1)
            hit, base = 0, 0
            for ref_sentence in reference:
                hit_mask = [0] * len(ref_sentence)
                base += len(ref_sentence)
                for model_sentence in summary:
                    self._longest_common_substring(ref_sentence, model_sentence, hit_mask)

                for i, token in enumerate(ref_sentence):
                    if hit_mask[i] == 1:
                        try:
                            if temp_model_unigrams[token] > 0 and gold_unigrams[token] > 0:
                                hit += 1
                                temp_model_unigrams[token] -= 1
                                gold_unigrams[token] -= 1
                        except KeyError:
                            pass
            total_hit += hit
            total_base += base

        precision = 0.0
        if (num_model_unigrams * len(references)) != 0.0:
            precision = total_hit / (num_model_unigrams * len(references)) * 100
        recall = 0.0
        if total_base != 0.0:
            recall = total_hit / total_base * 100
        if (precision + recall) != 0.0:
            f1 = 2 * (precision * recall) / (precision + recall)
        else:
            f1 = 0.0
        return precision, recall, f1

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        summaries_list = [[self.preprocess_summary(summary) for summary in summaries] for summaries in summaries_list]
        references_list = [[self.preprocess_summary(reference) for reference in references] for references in references_list]

        metrics_lists = []
        for summaries, references in zip(summaries_list, references_list):
            metrics_list = [MetricsDict() for _ in summaries]

            for n in self.ngram_orders:
                reference_ngrams_list = [self._count_ngrams(reference, n) for reference in references]

                for i, summary in enumerate(summaries):
                    total_reference_count = 0
                    total_summary_count = 0
                    total_intersection = 0

                    summary_ngrams = self._count_ngrams(summary, n)
                    for reference_ngrams in reference_ngrams_list:
                        reference_total, summary_total, intersection = self._calculate_intersection(reference_ngrams, summary_ngrams)

                        total_reference_count += reference_total
                        total_summary_count += summary_total
                        total_intersection += intersection

                    precision, recall, f1 = self._calculate_pr_f1(total_reference_count, total_summary_count, total_intersection)
                    metrics_list[i][f'python-rouge-{n}'] = {
                        'precision': precision,
                        'recall': recall,
                        'f1': f1,
                    }

            if self.compute_rouge_l:
                for i, summary in enumerate(summaries):
                    precision, recall, f1 = self._calculate_rouge_l(references, summary)
                    metrics_list[i]['python-rouge-l'] = {
                        'precision': precision,
                        'recall': recall,
                        'f1': f1
                    }

            metrics_lists.append(metrics_list)
        return metrics_lists


@MetricSetupSubcommand.register('python-rouge')
class PythonRougeSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the Python-based ROGUE metric'
        self.parser = parser.add_parser('python-rouge', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        print('Please run the ROUGE setup code instead.')


import lxml.html
import re
import tarfile
from collections import defaultdict
from io import BytesIO
from nltk.tokenize import sent_tokenize
from typing import Dict, List, Tuple

from sacrerouge.io import JsonlWriter


# There are errors in the extractive MDS that have the
# incorrect file IDs
doc_id_overrides = {
    'AP890729-0155': 'AP880729-0155',
    'SJMN91-06048': 'SJMN91-06165048',
    'AP880428-\r\n0041': 'AP880428-0041',
    'AP880818-\r\n0088': 'AP880818-0088'
}


def load_extractive_document(tar, member) -> List[str]:
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    sentences = []
    for node in tree.xpath('//s'):
        num = int(node.get('num'))
        assert num == len(sentences) + 1
        text = node.text_content().strip()
        sentences.append(text)
    return sentences


def load_abstractive_document(tar, member) -> List[str]:
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    sentences = []
    for node in tree.xpath('//text//s'):
        text = node.text_content().strip()
        sentences.append(text)
    assert len(sentences) > 0, member.name
    return sentences


def load_documents(documents_tar_path: str) -> Tuple[Dict[List, str], Dict[str, List[str]]]:
    cluster_to_documents = defaultdict(list)
    abs_documents = {}
    ext_documents = {}

    with tarfile.open(documents_tar_path, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2002_Summarization_Documents/duc2002testdocswithsentences.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                if member.isfile():
                    path = member.name.split('/')
                    cluster = path[2][:-1]
                    filename = path[3][:-2]

                    abs_sentences = load_abstractive_document(inner_tar, member)
                    ext_sentences = load_extractive_document(inner_tar, member)

                    abs_document = {'filename': filename, 'text': abs_sentences}
                    ext_document = {'filename': filename, 'text': ext_sentences}

                    cluster_to_documents[cluster].append(filename)
                    abs_documents[filename] = abs_document
                    ext_documents[filename] = ext_document

    return cluster_to_documents, abs_documents, ext_documents


def load_sds_summaries(summaries_tar: str) -> Dict[str, List[List[str]]]:
    summaries = defaultdict(list)
    with tarfile.open(summaries_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile():
                if member.name.startswith('./extracts_abstracts/d'):
                    path = member.name.split('/')
                    filename = path[-1]
                    if filename == 'perdocs':
                        html = tar.extractfile(member).read().decode()
                        tree = lxml.html.document_fromstring(html)
                        for node in tree.xpath('body/sum'):
                            doc = node.attrib['docref']
                            text = node.text_content().strip()
                            text = re.sub('\s+', ' ', text)
                            sentences = sent_tokenize(text)
                            annotator = path[2][-1].upper()
                            summary = {
                                'annotator': annotator,
                                'text': sentences
                            }
                            if doc in summaries and sentences in summaries[doc]:
                                print(f'Document {doc} has a duplicate SDS. Skipping')
                            summaries[doc].append(summary)
    return summaries


def load_mds_abstractive_summaries(summaries_tar: str) -> Dict[str, Dict[int, List[List[str]]]]:
    summaries = defaultdict(lambda: defaultdict(list))
    with tarfile.open(summaries_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile():
                if member.name.startswith('./extracts_abstracts/d'):
                    path = member.name.split('/')
                    cluster = path[2][:-2]
                    filename = path[-1]
                    if filename in ['10', '50', '100', '200']:
                        length = int(filename)
                        html = tar.extractfile(member).read().decode()
                        tree = lxml.html.document_fromstring(html)
                        text = tree.xpath('body/sum')[0].text_content().strip()
                        text = re.sub('\s+', ' ', text)
                        sentences = sent_tokenize(text)
                        annotator = path[2][-1].upper()
                        summary = {
                            'annotator': annotator,
                            'text': sentences
                        }
                        if sentences not in summaries[cluster][length]:
                            summaries[cluster][length].append(summary)
                        else:
                            print(f'Duplicate summary of length {length} found for cluster {cluster}')
    return summaries


def load_mds_extractive_summaries(summaries_tar: str) -> Dict[str, Dict[int, List[List[str]]]]:
    summaries = defaultdict(lambda: defaultdict(list))
    with tarfile.open(summaries_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile():
                if member.name.startswith('./extracts_abstracts/d'):
                    path = member.name.split('/')
                    cluster = path[2][:-2]
                    filename = path[-1]
                    if filename in ['200e', '400e']:
                        length = int(filename[:-1])
                        html = tar.extractfile(member).read().decode()
                        # There is a typo in this particular file where the closing
                        # tag is actually an opening tag, and it messes up the parse
                        if member.name.endswith('d118if/200e'):
                            html = html.replace('of <s>', 'of </s>')
                        tree = lxml.html.document_fromstring(html)
                        labels = []
                        for node in tree.xpath('//s'):
                            doc = node.get('docid')
                            num = int(node.get('num'))
                            index = num - 1
                            labels.append((doc, index))

                        annotator = path[2][-1].upper()
                        summary = {
                            'annotator': annotator,
                            'labels': labels
                        }

                        if labels in summaries[cluster][length]:
                            print(f'Cluster {cluster} has duplicate extractive summaries of length {length}')
                        else:
                            summaries[cluster][length].append(summary)
    return summaries


def save_sds_data(documents: Dict[str, List[str]],
                  summaries: Dict[str, List[List[str]]],
                  file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for doc_id, document in documents.items():
            doc_summaries = summaries[doc_id]
            data = {
                'instance_id': doc_id,
                'document': document,
                'summaries': doc_summaries
            }
            out.write(data)


def save_mds_abstractive_data(cluster_to_documents: Dict[str, List[str]],
                              documents: Dict[str, List[str]],
                              summaries: Dict[str, Dict[int, List[List[str]]]],
                              length: int,
                              file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for cluster_id in sorted(cluster_to_documents.keys()):
            doc_ids = cluster_to_documents[cluster_id]
            cluster_docs = [documents[doc_id] for doc_id in doc_ids]
            cluster_summaries = summaries[cluster_id][length]
            data = {
                'instance_id': cluster_id,
                'documents': cluster_docs,
                'summaries': cluster_summaries
            }
            out.write(data)


def save_mds_extractive_data(cluster_to_documents: Dict[str, List[str]],
                             documents: Dict[str, List[str]],
                             summaries: Dict[str, Dict[int, List[List[Tuple[str, int]]]]],
                             length: int,
                             file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for cluster_id in sorted(cluster_to_documents.keys()):
            doc_ids = cluster_to_documents[cluster_id]
            doc_id_to_index = {doc_id: i for i, doc_id in enumerate(doc_ids)}

            cluster_docs = [documents[doc_id] for doc_id in doc_ids]
            cluster_summaries = []
            for summary in summaries[cluster_id][length]:
                new_labels = []
                for doc_id, index in summary['labels']:
                    if doc_id in doc_id_overrides:
                        doc_id = doc_id_overrides[doc_id]
                    new_labels.append({
                        'doc_index': doc_id_to_index[doc_id],
                        'sent_index': index
                    })

                cluster_summaries.append({
                    'annotator': summary['annotator'],
                    'labels': new_labels
                })

            data = {
                'instance_id': cluster_id,
                'documents': cluster_docs,
                'summaries': cluster_summaries
            }
            out.write(data)


def setup(data_root: str, output_dir: str) -> None:
    documents_tar = f'{data_root}/from-nist/DUC2002_Summarization_Documents.tgz'
    extracts_and_abstracts_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2002/data/test/summaries/duc2002extractsandabstracts.tar.gz'
    main(documents_tar, extracts_and_abstracts_tar, output_dir)


def main(documents_tar, extracts_and_abstracts_tar, output_dir):
    cluster_to_documents, abs_documents, ext_documents = load_documents(documents_tar)
    sds_summaries = load_sds_summaries(extracts_and_abstracts_tar)
    mds_abs_summaries = load_mds_abstractive_summaries(extracts_and_abstracts_tar)
    mds_ext_summaries = load_mds_extractive_summaries(extracts_and_abstracts_tar)

    save_sds_data(abs_documents, sds_summaries, f'{output_dir}/task1.jsonl')

    save_mds_abstractive_data(cluster_to_documents, abs_documents, mds_abs_summaries, 10, f'{output_dir}/task2.10.jsonl')
    save_mds_abstractive_data(cluster_to_documents, abs_documents, mds_abs_summaries, 50, f'{output_dir}/task2.50.jsonl')
    save_mds_abstractive_data(cluster_to_documents, abs_documents, mds_abs_summaries, 100, f'{output_dir}/task2.100.jsonl')
    save_mds_abstractive_data(cluster_to_documents, abs_documents, mds_abs_summaries, 200, f'{output_dir}/task2.200.jsonl')

    save_mds_extractive_data(cluster_to_documents, ext_documents, mds_ext_summaries, 200, f'{output_dir}/task2.200e.jsonl')
    save_mds_extractive_data(cluster_to_documents, ext_documents, mds_ext_summaries, 400, f'{output_dir}/task2.400e.jsonl')



from typing import List

from sacrerouge.common import Registrable
from sacrerouge.data import EvalInstance


class DatasetReader(Registrable):
    def read(self, *args: List[str]) -> List[EvalInstance]:
        raise NotImplementedError



"""
Parses the DUC 2001 single- and multi-document summarization datasets. These
correspond to task 1 and task 2 from the competition. There are several peculiarities
about the dataset which we have tried to address, but it's possible we did not
get everything.
"""
import argparse
import lxml.html
import os
import re
import tarfile
from collections import defaultdict
from nltk.tokenize import sent_tokenize
from typing import Dict, List

from sacrerouge.io import JsonlWriter

# There are several typos in the 'perdocs' files in which the document
# id is incorrect. This provides a mapping from the incorrect id to the
# correct one
per_doc_summary_doc_id_fix = {
    'SMN91-06154062': 'SJMN91-06154062',
    'AP870611-0085': 'WSJ870611-0085',
    'SJMN91-0605144': 'SJMN91-06015144',
    'APP890515-0232': 'AP890515-0232',
    'WSJ891125-0090': 'AP891125-0090',
    'FT931-100514': 'FT931-10514',
    'FB153-57782': 'FBIS3-57782',
    'FBIS4-45908': 'FBIS-45908',
    'FBIS3-41815': 'FBIS-41815',
    'FBIS4-45908': 'FBIS-45908',
    'FBIS4-35908': 'FBIS-45908',
    'AP890325-0143': 'AP830325-0143',
    'LA081489-0225': 'LA081489-0025'
}

document_blacklist = [
    'FBIS-41815~'
]


def load_sjmn_document(tar, member) -> str:
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    text = []
    for node in tree.xpath('body/doc/leadpara | body/doc/text'):
        content = node.text_content().strip()

        # The SJMN files have sentences which end in '.;'. My guess is that
        # these mark paragraphs, but I don't know how to confirm that
        content = re.sub('\.;\s+', '. ', content)

        # There are a lot of quotes that end with semicolons that messes
        # up the sentence splitting logic
        content = re.sub('";', '"', content)

        # The newlines are meaningless since they often appear in the middle
        # of a sentence
        content = re.sub('\s+', ' ', content)
        text.append(content)

    text = ' '.join(text)
    return text


def load_wsj_document(tar, member) -> str:
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    text = []
    for node in tree.xpath('body/doc/lp | body/doc/text'):
        # The paragraphs seem to be indented with extra whitespace
        content = node.text_content().strip()
        content = re.sub('\s+', ' ', content)
        text.append(content)
    text = ' '.join(text)
    return text


def load_ap_document(tar, member) -> str:
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    # There is no lead paragraph for the AP documents.
    body = tree.xpath('body/doc/text')[0].text_content().strip()

    # The paragraphs seem to be indented with extra whitespace
    body = re.sub('\s+', ' ', body)
    return body


def load_la_document(tar, member) -> str:
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    body = tree.xpath('body/doc/text')[0].text_content().strip()

    # The paragraphs are separated by <P> tags, but the `text_content` function
    # retrieves all of the text from the descendants, so those are removed
    body = re.sub('\s+', ' ', body)
    return body


def load_fbis_document(tar, member) -> str:
    # The FBIS documents often have some metadata at the beginning of the
    # <text> tag which isn't removed here
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    body = tree.xpath('body/doc/text')[0].text_content().strip()

    # The paragraphs seem to be indented with extra whitespace
    body = re.sub('\s+', ' ', body)
    return body


def load_ft_document(tar, member) -> str:
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    body = tree.xpath('body/doc/text')[0].text_content().strip()

    # The paragraphs seem to be indented with extra whitespace
    body = re.sub('\s+', ' ', body)
    return body


def load_document(tar, member) -> List[str]:
    filename = os.path.basename(member.name)
    if filename.startswith('SJMN'):
        text = load_sjmn_document(tar, member)
    elif filename.startswith('WSJ'):
        text = load_wsj_document(tar, member)
    elif filename.startswith('AP'):
        text = load_ap_document(tar, member)
    elif filename.startswith('LA'):
        text = load_la_document(tar, member)
    elif filename.startswith('FBIS'):
        text = load_fbis_document(tar, member)
    elif filename.startswith('FT'):
        text = load_ft_document(tar, member)
    else:
        raise Exception(member.name)

    # Many quotes have '. . .' which mess up the sentence splitting
    text = re.sub(r'\. \. \.', '...', text)

    sentences = sent_tokenize(text)
    return {
        'filename': filename,
        'text': sentences
    }


def load_mds_summary(tar, member) -> List[str]:
    """
    Loads a multi-document summary from the file path and returns the
    sentence-tokenized summary.
    """
    html = tar.extractfile(member).read().decode()
    tree = lxml.html.document_fromstring(html)
    text = tree.xpath('body/sum')[0].text_content().strip()
    text = re.sub('\s+', ' ', text)
    sentences = sent_tokenize(text)

    dirname = os.path.basename(os.path.dirname(member.name))
    annotator = dirname[-1].upper()
    return {
        'annotator': annotator,
        'text': sentences
    }


def load_sds_summaries(tar, member) -> Dict[str, List[str]]:
    """
    Loads all of the individual single-document summaries and returns them as
    a dictionary mapping from the document ID to the sentence-tokenized summary.
    """
    html = tar.extractfile(member).read().decode()

    # The perdocs file for d35ff is missing a closing angle bracket
    # on one of the tags, which messes up the parser
    if os.path.basename(os.path.dirname(member.name)) == 'd35ff':
        html = html[:768] + '>' + html[768:]
    tree = lxml.html.document_fromstring(html)

    dirname = os.path.basename(os.path.dirname(member.name))
    annotator = dirname[-1].upper()

    summaries = {}
    for node in tree.xpath('body/sum'):
        doc = node.attrib['docref']
        if doc in per_doc_summary_doc_id_fix:
            doc = per_doc_summary_doc_id_fix[doc]

        text = node.text_content().strip()
        text = re.sub('\s+', ' ', text)
        sentences = sent_tokenize(text)
        summaries[doc] = {
            'annotator': annotator,
            'text': sentences
        }
    return summaries


def load_training_data(documents_tar: str):
    # ``cluster_to_doc_ids`` maps from the cluster name to the list of
    # document IDs that belong to the cluster
    cluster_to_doc_ids = defaultdict(list)
    # ``documents`` maps from the document ID to the ``List[str]`` document
    documents = {}
    # ``mds_summaries`` maps from the cluster name then summary length to the summary
    mds_summaries = defaultdict(lambda: defaultdict(list))
    # ``sds_summaries`` maps from the file name to the summary
    sds_summaries = defaultdict(list)

    with tarfile.open(documents_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile():
                if member.name.startswith('DUC2001_Summarization_Documents/data/training'):
                    path = member.name.split('/')
                    cluster = path[3][:-1]
                    filename = path[-1]

                    if filename == 'perdocs':
                        summaries = load_sds_summaries(tar, member)
                        for doc_id in sorted(summaries.keys()):
                            sds_summaries[doc_id].append(summaries[doc_id])
                    elif filename in ['50', '100', '200', '400']:
                        summary = load_mds_summary(tar, member)
                        mds_summaries[cluster][filename].append(summary)
                    elif len(path) >= 5 and path[4] == 'docs':
                        document = load_document(tar, member)
                        cluster_to_doc_ids[cluster].append(filename)
                        documents[filename] = document

    return cluster_to_doc_ids, documents, mds_summaries, sds_summaries


def load_test_data(documents_tar: str):
    # ``cluster_to_doc_ids`` maps from the cluster name to the list of
    # document IDs that belong to the cluster
    cluster_to_doc_ids = defaultdict(list)
    # ``documents`` maps from the document ID to the ``List[str]`` document
    documents = {}
    # ``mds_summaries`` maps from the cluster name then summary length to the summaries
    mds_summaries = defaultdict(lambda: defaultdict(list))
    # ``sds_summaries`` maps from the file name to the summary
    sds_summaries = defaultdict(list)

    with tarfile.open(documents_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile():
                if member.name.startswith('DUC2001_Summarization_Documents/data/test/docs'):
                    path = member.name.split('/')
                    if len(path) != 6:
                        continue
                    cluster = path[-2][:-1]
                    doc_id = path[-1]
                    if doc_id in document_blacklist:
                        continue
                    cluster_to_doc_ids[cluster].append(doc_id)

                    document = load_document(tar, member)
                    documents[doc_id] = document
                elif member.name.startswith('DUC2001_Summarization_Documents/data/test/original.summaries') or member.name.startswith('DUC2001_Summarization_Documents/data/test/duplicate.summaries'):
                    path = member.name.split('/')
                    if len(path) != 6:
                        continue
                    cluster = path[-2][:-2]
                    filename = path[-1]
                    if filename == 'perdocs':
                        summaries = load_sds_summaries(tar, member)
                        for doc_id in sorted(summaries.keys()):
                            sds_summaries[doc_id].append(summaries[doc_id])
                    elif filename in ['50', '100', '200', '400']:
                        summary = load_mds_summary(tar, member)
                        mds_summaries[cluster][filename].append(summary)

    return cluster_to_doc_ids, documents, mds_summaries, sds_summaries


def save_sds_training_data(documents: Dict[str, List[str]],
                           summaries: Dict[str, List[List[str]]],
                           file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        assert set(documents.keys()) == set(summaries.keys())
        for doc_id in sorted(documents.keys()):
            document = documents[doc_id]
            doc_summaries = summaries[doc_id]
            # This document appears in two clusters and so it has two
            # single document summaries
            if doc_id == 'SJMN91-06170171':
                assert len(doc_summaries) == 2
            else:
                assert len(doc_summaries) == 1
            data = {
                'instance_id': doc_id,
                'document': document,
                'summaries': doc_summaries
            }
            out.write(data)


def save_sds_test_data(documents: Dict[str, List[str]],
                       summaries: Dict[str, List[List[str]]],
                       file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for doc_id in sorted(documents.keys()):
            document = documents[doc_id]
            doc_summaries = summaries[doc_id]
            # The original summaries for cluster d14 is missing a summary
            # for document FBIS4-4674 and likewise for dd44 and FT934-10911
            if doc_id in ['FBIS4-4674', 'FT934-10911']:
                assert len(doc_summaries) == 2
            else:
                assert len(doc_summaries) == 3, doc_id
            data = {
                'instance_id': doc_id,
                'document': document,
                'summaries': doc_summaries
            }
            out.write(data)


def save_mds_training_data(cluster_to_doc_ids: Dict[str, List[str]],
                           documents: Dict[str, List[str]],
                           summaries: Dict[str, Dict[str, List[str]]],
                           length: str,
                           file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        assert set(cluster_to_doc_ids.keys()) == set(summaries.keys())
        for cluster in sorted(cluster_to_doc_ids.keys()):
            doc_ids = cluster_to_doc_ids[cluster]
            cluster_documents = [documents[doc_id] for doc_id in sorted(doc_ids)]
            summary = summaries[cluster][length]
            data = {
                'instance_id': cluster,
                'documents': cluster_documents,
                'summaries': [summary]
            }
            out.write(data)


def save_mds_test_data(cluster_to_doc_ids: Dict[str, List[str]],
                       documents: Dict[str, List[str]],
                       summaries: Dict[str, Dict[str, List[str]]],
                       length: str,
                       file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for cluster in sorted(cluster_to_doc_ids.keys()):
            doc_ids = cluster_to_doc_ids[cluster]
            cluster_documents = [documents[doc_id] for doc_id in sorted(doc_ids)]
            cluster_summaries = summaries[cluster][length]
            if cluster == 'd31':
                assert len(cluster_summaries) == 2
            else:
                assert len(cluster_summaries) == 3
            data = {
                'instance_id': cluster,
                'documents': cluster_documents,
                'summaries': cluster_summaries
            }
            out.write(data)


def setup(data_root: str, output_dir: str) -> None:
    documents_tar = f'{data_root}/from-nist/DUC2001_Summarization_Documents.tgz'
    main(documents_tar, output_dir)


def main(documents_tar, output_dir):
    cluster_to_doc_ids, documents, mds_summaries, sds_summaries \
        = load_training_data(documents_tar)

    train_sds_path = f'{output_dir}/task1.train.jsonl'
    save_sds_training_data(documents, sds_summaries, train_sds_path)

    train_mds_50_path = f'{output_dir}/task2.train.50.jsonl'
    train_mds_100_path = f'{output_dir}/task2.train.100.jsonl'
    train_mds_200_path = f'{output_dir}/task2.train.200.jsonl'
    train_mds_400_path = f'{output_dir}/task2.train.400.jsonl'
    save_mds_training_data(cluster_to_doc_ids, documents, mds_summaries, "50", train_mds_50_path)
    save_mds_training_data(cluster_to_doc_ids, documents, mds_summaries, "100", train_mds_100_path)
    save_mds_training_data(cluster_to_doc_ids, documents, mds_summaries, "200", train_mds_200_path)
    save_mds_training_data(cluster_to_doc_ids, documents, mds_summaries, "400", train_mds_400_path)

    cluster_to_doc_ids, documents, mds_summaries, sds_summaries = \
        load_test_data(documents_tar)

    test_sds_path = f'{output_dir}/task1.test.jsonl'
    save_sds_test_data(documents, sds_summaries, test_sds_path)

    test_mds_50_path = f'{output_dir}/task2.test.50.jsonl'
    test_mds_100_path = f'{output_dir}/task2.test.100.jsonl'
    test_mds_200_path = f'{output_dir}/task2.test.200.jsonl'
    test_mds_400_path = f'{output_dir}/task2.test.400.jsonl'
    save_mds_test_data(cluster_to_doc_ids, documents, mds_summaries, "50", test_mds_50_path)
    save_mds_test_data(cluster_to_doc_ids, documents, mds_summaries, "100", test_mds_100_path)
    save_mds_test_data(cluster_to_doc_ids, documents, mds_summaries, "200", test_mds_200_path)
    save_mds_test_data(cluster_to_doc_ids, documents, mds_summaries, "400", test_mds_400_path)


import tarfile
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Pyramid, PyramidAnnotation
from sacrerouge.io import JsonlWriter


def load_pyramids(pyramid_tar: str) -> Dict[str, Pyramid]:
    pyramids = {}
    with tarfile.open(pyramid_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('pans/'):
                path = member.name.split('/')
                filename = path[-1]
                instance_id = filename.split('.')[1].lower()

                # For this dataset, the pyramid and annotations are all in the same file, one per
                # annotation. Therefore, we only need to load the pyramid once from the first file
                if instance_id in pyramids:
                    continue

                xml = tar.extractfile(member).read().decode()
                pyramid = Pyramid.from_xml(instance_id, xml,
                                           default_document_regex='[-]*\n(\s*)D[0-9]*\.M\.250\.[A-Z]\.[A-Z]\n[-]*\n',
                                           is_combined_file=True)
                pyramids[instance_id] = pyramid

    return pyramids


def load_peer_pyramids(eval_tar: str, pyramids: Dict[str, Pyramid]) -> Dict[str, Dict[str, PyramidAnnotation]]:
    annotations = defaultdict(dict)
    with tarfile.open(eval_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('pans/'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[1].lower()
                summarizer_id = parts[5]

                if summarizer_id.isalpha():
                    summarizer_type = 'reference'
                else:
                    summarizer_type = 'peer'

                pyramid = pyramids[instance_id]
                xml = tar.extractfile(member).read().decode()
                annotation = PyramidAnnotation.from_xml(f'{instance_id}', summarizer_id, summarizer_type, xml, pyramid)
                if annotation:
                    annotations[instance_id][summarizer_id] = annotation
                else:
                    print(f'Annotation for {instance_id}, {summarizer_id} is `None`. Skipping')

    return annotations


def save_pyramids(pyramids: Dict[str, Pyramid],
                  output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.pyramids.jsonl') as out:
        for instance_id in sorted(pyramids.keys()):
            pyramid = pyramids[instance_id]
            out.write(pyramid)


def save_peer_pyramids(annotations: Dict[str, Dict[str, PyramidAnnotation]],
                       output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.pyramid-annotations.jsonl') as out:
        for instance_id in sorted(annotations.keys()):
            for summarizer_id in sorted(annotations[instance_id].keys()):
                annotation = annotations[instance_id][summarizer_id]
                out.write(annotation)


def setup(data_dir: str, output_dir: str):
    pyramid_tar = f'{data_dir}/scrapes/duc.nist.gov/past_duc/duc2005/results/Pyramid/DUC2005.tar.gz'
    main(pyramid_tar, output_dir)


def main(pyramid_tar, output_dir):
    pyramids = load_pyramids(pyramid_tar)
    save_pyramids(pyramids, output_dir)

    peer_pyramids = load_peer_pyramids(pyramid_tar, pyramids)
    save_peer_pyramids(peer_pyramids, output_dir)




from sacrerouge.metrics.metric import DocumentBasedMetric, Metric, ReferenceBasedMetric, ReferenceFreeMetric, SummaryBasedMetric
from sacrerouge.metrics.autosummeng import AutoSummENG
from sacrerouge.metrics.bertscore import BertScore
from sacrerouge.metrics.bewte import BEwTE
from sacrerouge.metrics.bleurt import Bleurt
from sacrerouge.metrics.decomposed_rouge import DecomposedRouge
from sacrerouge.metrics.meteor import Meteor
from sacrerouge.metrics.moverscore import MoverScore
from sacrerouge.metrics.pyramid_score import PyramidScore
from sacrerouge.metrics.pyreval import PyrEval
from sacrerouge.metrics.python_rouge import PythonRouge
from sacrerouge.metrics.qaeval import QAEval
from sacrerouge.metrics.rouge import Rouge
from sacrerouge.metrics.s3 import S3
from sacrerouge.metrics.simetrix import SIMetrix
from sacrerouge.metrics.sumqe import SumQE
from sacrerouge.metrics.supert import SUPERT



import argparse
import json
import os
from collections import defaultdict


def get_datasets(args):
    datasets = []
    for _, _, dataset in args.input:
        if dataset not in datasets:
            datasets.append(dataset)
    return datasets


def get_metrics(args):
    metrics = []
    for _, metric, _ in args.input:
        if metric not in metrics:
            metrics.append(metric)
    return metrics


def main(args):
    datasets = get_datasets(args)
    metrics = get_metrics(args)

    values = defaultdict(dict)
    for file_path, metric, dataset in args.input:
        correlations = json.load(open(file_path, 'r'))
        values[metric][dataset] = correlations

    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(args.output_file, 'w') as out:
        out.write('<table>\n')

        # Print the header with the dataset names
        out.write('<tr>\n')
        out.write('<th></th>\n')
        for dataset in datasets:
            out.write(f'<th colspan="3">{dataset}</th>\n')
        out.write('</tr>\n')

        # Print the header with the correlation coefficient names
        out.write('<tr>\n')
        out.write('<th></th>\n')
        for dataset in datasets:
            for coef in ['r', 'p', 'k']:
                out.write(f'<th>{coef}</th>\n')
        out.write('</tr>\n')

        # Print each value in the table
        for metric in metrics:
            out.write('<tr>\n')
            out.write(f'<td>{metric}</td>\n')
            for dataset in datasets:
                out.write(f'<td>{values[metric][dataset][args.correlation_level]["pearson"]["r"]:.2f}</td>\n')
                out.write(f'<td>{values[metric][dataset][args.correlation_level]["spearman"]["rho"]:.2f}</td>\n')
                out.write(f'<td>{values[metric][dataset][args.correlation_level]["kendall"]["tau"]:.2f}</td>\n')
            out.write('</tr>\n')

        out.write('</table>\n')


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('output_file')
    argp.add_argument('correlation_level', choices=['summary_level', 'system_level', 'global'])
    argp.add_argument('--input', action='append', nargs=3, metavar=('file_path', 'metric', 'dataset'))
    args = argp.parse_args()
    main(args)


from sacrerouge.metrics.decomposed_rouge.categorizers.categorizer import Categorizer, TupleCategorizer
from sacrerouge.metrics.decomposed_rouge.categorizers.dep import DependencyCategorizer, DependencyVerbRelationsCategorizer
from sacrerouge.metrics.decomposed_rouge.categorizers.ner import NERCategorizer
from sacrerouge.metrics.decomposed_rouge.categorizers.np_chunks import NPChunkCategorizer
from sacrerouge.metrics.decomposed_rouge.categorizers.pos import POSCategorizer
from sacrerouge.metrics.decomposed_rouge.categorizers.stopwords import StopwordCategorizer



import os
import pytest
import unittest

from sacrerouge.commands.correlate import aggregate_metrics
from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_metrics_A_file_path = 'datasets/duc-tac/tac2009/v1.0/task1.A.metrics.jsonl'
_metrics_B_file_path = 'datasets/duc-tac/tac2009/v1.0/task1.B.metrics.jsonl'


class TestTAC2009SystemLevel(unittest.TestCase):
    @pytest.mark.skipif(not os.path.exists(_metrics_A_file_path), reason='TAC 2009-A metrics file does not exist')
    def test_system_level_A(self):
        summary_level_metrics = JsonlReader(_metrics_A_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2_A.m.avg
        assert system_level_metrics['2']['rouge-2']['recall'] == pytest.approx(33.165, 1e-2)
        assert system_level_metrics['34']['rouge-2']['recall'] == pytest.approx(12.163, 1e-2)
        assert system_level_metrics['40']['rouge-2']['recall'] == pytest.approx(12.089, 1e-2)
        assert system_level_metrics['35']['rouge-2']['recall'] == pytest.approx(10.869, 1e-2)
        assert system_level_metrics['3']['rouge-2']['recall'] == pytest.approx(10.655, 1e-2)

        # ROUGE/rouge2_A.jk.m.avg
        assert system_level_metrics['C']['rouge-2_jk']['recall'] == pytest.approx(14.864, 1e-2)
        assert system_level_metrics['H']['rouge-2_jk']['recall'] == pytest.approx(13.457, 1e-2)
        assert system_level_metrics['E']['rouge-2_jk']['recall'] == pytest.approx(13.341, 1e-2)
        assert system_level_metrics['2']['rouge-2_jk']['recall'] == pytest.approx(33.133, 1e-2)
        assert system_level_metrics['34']['rouge-2_jk']['recall'] == pytest.approx(12.184, 1e-2)

        # ROUGE/rougeSU4_A.m.avg
        assert system_level_metrics['2']['rouge-su4']['recall'] == pytest.approx(34.421, 1e-2)
        assert system_level_metrics['40']['rouge-su4']['recall'] == pytest.approx(15.101, 1e-2)
        assert system_level_metrics['34']['rouge-su4']['recall'] == pytest.approx(15.030, 1e-2)
        assert system_level_metrics['35']['rouge-su4']['recall'] == pytest.approx(14.487, 1e-2)
        assert system_level_metrics['51']['rouge-su4']['recall'] == pytest.approx(14.165, 1e-2)

        # ROUGE/rougeSU4_A.jk.m.avg
        assert system_level_metrics['C']['rouge-su4_jk']['recall'] == pytest.approx(18.355, 1e-2)
        assert system_level_metrics['H']['rouge-su4_jk']['recall'] == pytest.approx(17.199, 1e-2)
        assert system_level_metrics['E']['rouge-su4_jk']['recall'] == pytest.approx(16.917, 1e-2)
        assert system_level_metrics['2']['rouge-su4_jk']['recall'] == pytest.approx(34.399, 1e-2)
        assert system_level_metrics['40']['rouge-su4_jk']['recall'] == pytest.approx(15.131, 1e-2)

        # manual/manual.model.A.avg
        assert system_level_metrics['A']['num_scus_jk'] == pytest.approx(10.364, 1e-2)
        assert system_level_metrics['B']['num_scus_jk'] == pytest.approx(9.5, 1e-2)
        assert system_level_metrics['C']['num_scus_jk'] == pytest.approx(12.364, 1e-2)

        assert system_level_metrics['A']['modified_pyramid_score_jk'] == pytest.approx(0.685, 1e-2)
        assert system_level_metrics['B']['modified_pyramid_score_jk'] == pytest.approx(0.616, 1e-2)
        assert system_level_metrics['C']['modified_pyramid_score_jk'] == pytest.approx(0.720, 1e-2)

        assert system_level_metrics['A']['linguistic_quality'] == pytest.approx(8.636, 1e-2)
        assert system_level_metrics['B']['linguistic_quality'] == pytest.approx(9.136, 1e-2)
        assert system_level_metrics['C']['linguistic_quality'] == pytest.approx(9.136, 1e-2)

        assert system_level_metrics['A']['overall_responsiveness'] == pytest.approx(8.455, 1e-2)
        assert system_level_metrics['B']['overall_responsiveness'] == pytest.approx(8.727, 1e-2)
        assert system_level_metrics['C']['overall_responsiveness'] == pytest.approx(9.318, 1e-2)

        # manual/manual.peer.A.avg
        assert system_level_metrics['1']['modified_pyramid_score'] == pytest.approx(0.175, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score'] == pytest.approx(0.646, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score'] == pytest.approx(0.358, 1e-2)

        assert system_level_metrics['1']['num_scus'] == pytest.approx(3.182, 1e-2)
        assert system_level_metrics['2']['num_scus'] == pytest.approx(11.977, 1e-2)
        assert system_level_metrics['3']['num_scus'] == pytest.approx(6.00, 1e-2)

        assert system_level_metrics['1']['num_repetitions'] == pytest.approx(1.318, 1e-2)
        assert system_level_metrics['2']['num_repetitions'] == pytest.approx(2.455, 1e-2)
        assert system_level_metrics['3']['num_repetitions'] == pytest.approx(1.568, 1e-2)

        assert system_level_metrics['1']['modified_pyramid_score_jk'] == pytest.approx(0.172, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score_jk'] == pytest.approx(0.635, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score_jk'] == pytest.approx(0.352, 1e-2)

        assert system_level_metrics['1']['linguistic_quality'] == pytest.approx(6.705, 1e-2)
        assert system_level_metrics['2']['linguistic_quality'] == pytest.approx(5.477, 1e-2)
        assert system_level_metrics['3']['linguistic_quality'] == pytest.approx(7.477, 1e-2)

        assert system_level_metrics['1']['overall_responsiveness'] == pytest.approx(3.636, 1e-2)
        assert system_level_metrics['2']['overall_responsiveness'] == pytest.approx(6.364, 1e-2)
        assert system_level_metrics['3']['overall_responsiveness'] == pytest.approx(6.341, 1e-2)

        # BE/simple_A.m.hm.avg
        assert system_level_metrics['2']['rouge-be-hm']['recall'] == pytest.approx(24.820, 1e-2)
        assert system_level_metrics['34']['rouge-be-hm']['recall'] == pytest.approx(6.356, 1e-2)
        assert system_level_metrics['40']['rouge-be-hm']['recall'] == pytest.approx(6.321, 1e-2)
        assert system_level_metrics['45']['rouge-be-hm']['recall'] == pytest.approx(5.899, 1e-2)
        assert system_level_metrics['4']['rouge-be-hm']['recall'] == pytest.approx(5.843, 1e-2)

        # BE/simplejk_A.m.hm.avg
        assert system_level_metrics['C']['rouge-be-hm_jk']['recall'] == pytest.approx(7.876, 1e-2)
        assert system_level_metrics['E']['rouge-be-hm_jk']['recall'] == pytest.approx(6.909, 1e-2)
        assert system_level_metrics['F']['rouge-be-hm_jk']['recall'] == pytest.approx(6.840, 1e-2)
        assert system_level_metrics['2']['rouge-be-hm_jk']['recall'] == pytest.approx(24.830, 1e-2)
        assert system_level_metrics['34']['rouge-be-hm_jk']['recall'] == pytest.approx(6.379, 1e-2)

        # aesop_allpeers_A
        assert system_level_metrics['A']['aesop']['1'] == pytest.approx(0.154895909090909, 1e-2)
        assert system_level_metrics['C']['aesop']['8'] == pytest.approx(0.0419939626932389, 1e-2)
        assert system_level_metrics['4']['aesop']['13'] == pytest.approx(0.2186197727, 1e-2)
        assert system_level_metrics['8']['aesop']['22'] == pytest.approx(0.1286081818, 1e-2)
        assert system_level_metrics['16']['aesop']['30'] == pytest.approx(0.2341865909, 1e-2)

    @pytest.mark.skipif(not os.path.exists(_metrics_B_file_path), reason='TAC 2009-B metrics file does not exist')
    def test_system_level_B(self):
        summary_level_metrics = JsonlReader(_metrics_B_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2_B.m.avg
        assert system_level_metrics['2']['rouge-2']['recall'] == pytest.approx(31.956, 1e-2)
        assert system_level_metrics['34']['rouge-2']['recall'] == pytest.approx(10.386, 1e-2)
        assert system_level_metrics['40']['rouge-2']['recall'] == pytest.approx(10.373, 1e-2)
        assert system_level_metrics['35']['rouge-2']['recall'] == pytest.approx(10.104, 1e-2)
        assert system_level_metrics['3']['rouge-2']['recall'] == pytest.approx(9.820, 1e-2)

        # ROUGE/rouge2_B.jk.m.avg
        # C is off by a bit?
        # assert system_level_metrics['C']['rouge-2_jk']['recall'] == pytest.approx(12.550, 1e-2)
        assert system_level_metrics['H']['rouge-2_jk']['recall'] == pytest.approx(12.436, 1e-2)
        assert system_level_metrics['E']['rouge-2_jk']['recall'] == pytest.approx(11.001, 1e-2)
        assert system_level_metrics['2']['rouge-2_jk']['recall'] == pytest.approx(31.932, 1e-2)
        assert system_level_metrics['34']['rouge-2_jk']['recall'] == pytest.approx(10.417, 1e-2)

        # ROUGE/rougeSU4_B.m.avg
        assert system_level_metrics['2']['rouge-su4']['recall'] == pytest.approx(33.688, 1e-2)
        assert system_level_metrics['40']['rouge-su4']['recall'] == pytest.approx(13.948, 1e-2)
        assert system_level_metrics['34']['rouge-su4']['recall'] == pytest.approx(13.851, 1e-2)
        assert system_level_metrics['35']['rouge-su4']['recall'] == pytest.approx(13.839, 1e-2)
        assert system_level_metrics['51']['rouge-su4']['recall'] == pytest.approx(13.650, 1e-2)

        # ROUGE/rougeSU4_B.jk.m.avg
        assert system_level_metrics['C']['rouge-su4_jk']['recall'] == pytest.approx(16.386, 1e-2)
        assert system_level_metrics['H']['rouge-su4_jk']['recall'] == pytest.approx(16.602, 1e-2)
        assert system_level_metrics['E']['rouge-su4_jk']['recall'] == pytest.approx(15.152, 1e-2)
        assert system_level_metrics['2']['rouge-su4_jk']['recall'] == pytest.approx(33.668, 1e-2)
        assert system_level_metrics['40']['rouge-su4_jk']['recall'] == pytest.approx(13.959, 1e-2)

        # manual/manual.model.B.avg
        assert system_level_metrics['A']['num_scus_jk'] == pytest.approx(6.455, 1e-2)
        assert system_level_metrics['B']['num_scus_jk'] == pytest.approx(8.591, 1e-2)
        assert system_level_metrics['C']['num_scus_jk'] == pytest.approx(8.545, 1e-2)

        assert system_level_metrics['A']['modified_pyramid_score_jk'] == pytest.approx(0.481, 1e-2)
        assert system_level_metrics['B']['modified_pyramid_score_jk'] == pytest.approx(0.663, 1e-2)
        assert system_level_metrics['C']['modified_pyramid_score_jk'] == pytest.approx(0.640, 1e-2)

        assert system_level_metrics['A']['linguistic_quality'] == pytest.approx(8.727, 1e-2)
        assert system_level_metrics['B']['linguistic_quality'] == pytest.approx(8.545, 1e-2)
        assert system_level_metrics['C']['linguistic_quality'] == pytest.approx(9.364, 1e-2)

        assert system_level_metrics['A']['overall_responsiveness'] == pytest.approx(8.364, 1e-2)
        assert system_level_metrics['B']['overall_responsiveness'] == pytest.approx(8.318, 1e-2)
        assert system_level_metrics['C']['overall_responsiveness'] == pytest.approx(9.136, 1e-2)

        # manual/manual.peer.B.avg
        assert system_level_metrics['1']['modified_pyramid_score'] == pytest.approx(0.160, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score'] == pytest.approx(0.690, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score'] == pytest.approx(0.329, 1e-2)

        assert system_level_metrics['1']['num_scus'] == pytest.approx(2.386, 1e-2)
        assert system_level_metrics['2']['num_scus'] == pytest.approx(9.886, 1e-2)
        assert system_level_metrics['3']['num_scus'] == pytest.approx(4.545, 1e-2)

        assert system_level_metrics['1']['num_repetitions'] == pytest.approx(0.841, 1e-2)
        assert system_level_metrics['2']['num_repetitions'] == pytest.approx(1.955, 1e-2)
        assert system_level_metrics['3']['num_repetitions'] == pytest.approx(0.955, 1e-2)

        assert system_level_metrics['1']['modified_pyramid_score_jk'] == pytest.approx(0.158, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score_jk'] == pytest.approx(0.677, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score_jk'] == pytest.approx(0.324, 1e-2)

        assert system_level_metrics['1']['linguistic_quality'] == pytest.approx(6.455, 1e-2)
        assert system_level_metrics['2']['linguistic_quality'] == pytest.approx(5.886, 1e-2)
        assert system_level_metrics['3']['linguistic_quality'] == pytest.approx(7.250, 1e-2)

        assert system_level_metrics['1']['overall_responsiveness'] == pytest.approx(4.318, 1e-2)
        assert system_level_metrics['2']['overall_responsiveness'] == pytest.approx(6.182, 1e-2)
        assert system_level_metrics['3']['overall_responsiveness'] == pytest.approx(6.114, 1e-2)

        # BE/simple_B.m.hm.avg
        assert system_level_metrics['2']['rouge-be-hm']['recall'] == pytest.approx(25.041, 1e-2)
        assert system_level_metrics['24']['rouge-be-hm']['recall'] == pytest.approx(6.389, 1e-2)
        assert system_level_metrics['40']['rouge-be-hm']['recall'] == pytest.approx(6.162, 1e-2)
        assert system_level_metrics['34']['rouge-be-hm']['recall'] == pytest.approx(6.118, 1e-2)
        assert system_level_metrics['35']['rouge-be-hm']['recall'] == pytest.approx(5.813, 1e-2)

        # BE/simplejk_B.m.hm.avg
        # F is off by a little
        assert system_level_metrics['C']['rouge-be-hm_jk']['recall'] == pytest.approx(6.795, 1e-2)
        assert system_level_metrics['H']['rouge-be-hm_jk']['recall'] == pytest.approx(7.040, 1e-2)
        assert system_level_metrics['F']['rouge-be-hm_jk']['recall'] == pytest.approx(6.094, 1e-1)
        assert system_level_metrics['2']['rouge-be-hm_jk']['recall'] == pytest.approx(25.042, 1e-2)
        assert system_level_metrics['34']['rouge-be-hm_jk']['recall'] == pytest.approx(6.134, 1e-2)

        # aesop_allpeers_B
        assert system_level_metrics['B']['aesop']['2'] == pytest.approx(0.04890409091, 1e-2)
        assert system_level_metrics['E']['aesop']['4'] == pytest.approx(0.2740872727, 1e-2)
        assert system_level_metrics['6']['aesop']['7'] == pytest.approx(0.5850288957, 1e-2)
        assert system_level_metrics['9']['aesop']['20'] == pytest.approx(0.06261788636, 1e-2)
        assert system_level_metrics['14']['aesop']['34'] == pytest.approx(0.3664196656, 1e-2)



import argparse
from overrides import overrides
from typing import List

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.data import MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric

try:
    from bert_score import score, create_idf_dict
except ImportError:
    BERTSCORE_INSTALLED = False

    @Metric.register('bertscore')
    class BertScore(ReferenceBasedMetric):
        def __init__(self, *args, **kwargs):
            pass

        def score_multi_all(self, *args, **kwargs):
            raise NotImplementedError('Please install the "bert_score" python library library to use BertScore')
else:
    BERTSCORE_INSTALLED = True

    @Metric.register('bertscore')
    class BertScore(ReferenceBasedMetric):
        def __init__(self,
                     model_type: str = None,
                     num_layers: int = None,
                     idf: bool = False,
                     nthreads: int = 4,
                     batch_size: int = 64,
                     lang: str = 'en',
                     verbose: bool = False) -> None:
            super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())
            self.model_type = model_type
            self.num_layers = num_layers
            self.idf = idf
            self.nthreads = nthreads
            self.batch_size = batch_size
            self.lang = lang
            self.verbose = verbose

        def _flatten_summary(self, summary: SummaryType) -> str:
            if isinstance(summary, list):
                return ' '.join(summary)
            return summary

        def _get_unique_references(self, references_list: List[List[str]]) -> List[str]:
            unique_references = set()
            for references in references_list:
                for reference in references:
                    unique_references.add(reference)
            return list(unique_references)

        def _run(self,
                 summaries_list: List[List[SummaryType]],
                 references_list: List[List[SummaryType]]) -> List[List[MetricsDict]]:
            summaries_list = [[self._flatten_summary(summary) for summary in summaries] for summaries in summaries_list]
            references_list = [[self._flatten_summary(reference) for reference in references] for references in
                               references_list]

            # Create the candidate and reference lists for passing to the scoring function
            input_candidates = []
            input_references = []
            empty_inputs = set()
            for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
                for j, summary in enumerate(summaries):
                    if len(summary) == 0:
                        empty_inputs.add((i, j))
                    else:
                        input_candidates.append(summary)
                        input_references.append(references)

            # If we are using IDF weighting, then the default script will create
            # the IDF score based on all of the references, but we could potentially
            # have duplicate references, which would make "incorrect" counts. Instead,
            # create the idf dict based on unique references
            if self.idf:
                unique_references = self._get_unique_references(references_list)
                idf = create_idf_dict(unique_references, model_type=self.model_type, nthreads=self.nthreads,
                                      lang=self.lang)
            else:
                idf = False

            # Score the summaries
            precisions, recalls, f1s = score(input_candidates, input_references,
                                             model_type=self.model_type,
                                             num_layers=self.num_layers,
                                             idf=idf,
                                             nthreads=self.nthreads,
                                             batch_size=self.batch_size,
                                             lang=self.lang,
                                             verbose=self.verbose)

            # Remap the scores to the summaries
            index = 0
            metrics_lists = []
            for i, summaries in enumerate(summaries_list):
                metrics_lists.append([])
                for j, summary in enumerate(summaries):
                    if (i, j) in empty_inputs:
                        precision, recall, f1 = 0.0, 0.0, 0.0
                    else:
                        precision = precisions[index].item()
                        recall = recalls[index].item()
                        f1 = f1s[index].item()
                        index += 1

                    metrics_lists[-1].append(MetricsDict({
                        'bertscore': {
                            'precision': precision,
                            'recall': recall,
                            'f1': f1,
                        }
                    }))

            return metrics_lists

        def score_multi_all(self,
                            summaries_list: List[List[SummaryType]],
                            references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
            return self._run(summaries_list, references_list)


@MetricSetupSubcommand.register('bertscore')
class BertScoreSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the BERTScore metric'
        self.parser = parser.add_parser('bertscore', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        try:
            import bert_score
            print('BertScore setup success')
        except ImportError:
            print('Please pip install "bert_score" to complete setup')


import argparse
import distutils.util
import inspect
import json
from typing import Type

from sacrerouge.common import Params, Registrable
from sacrerouge.common.from_params import _NO_DEFAULT, construct_arg
from sacrerouge.data.dataset_readers import DatasetReader
from sacrerouge.metrics import Metric


def get_dataset_reader_from_argument(argument: str) -> DatasetReader:
    if argument in Registrable._registry[DatasetReader]:
        type_, _ = Registrable._registry[DatasetReader][argument]
        return type_()
    else:
        params = Params(json.loads(argument))
        return DatasetReader.from_params(params)


def add_metric_arguments(parser: argparse.ArgumentParser, metric_type: Type):
    """Inspect the `metric`'s constructor and add an argument to the `parser` for each parameter."""
    signature = inspect.signature(metric_type.__init__)
    for param_name in signature.parameters:
        name = signature.parameters[param_name].name
        if name == 'self':
            continue  # Ignore self in constructor
        default_value = signature.parameters[param_name].default
        annotation = signature.parameters[param_name].annotation

        kwargs = {}
        kwargs['required'] = default_value == signature.empty

        if annotation in [int, float, str]:
            kwargs['type'] = annotation
            if default_value != signature.empty:
                kwargs['default'] = default_value
        elif annotation is bool:
            kwargs['type'] = lambda arg: bool(distutils.util.strtobool(arg))
            if default_value != signature.empty:
                kwargs['default'] = default_value
        else:
            # There is either some type (but it's not a primitive (e.g. List[str])), or there's no typing information.
            # Either way, we assume the argument is a serialized json.
            kwargs['type'] = str
            if default_value is not None and default_value != signature.empty:
                kwargs['default'] = json.dumps(default_value)

        parser.add_argument(f'--{name}', **kwargs)


def get_metric_from_arguments(metric_type: Type, args: argparse.Namespace) -> Metric:
    kwargs = {}
    signature = inspect.signature(metric_type.__init__)
    for param_name in signature.parameters:
        name = signature.parameters[param_name].name
        if name == 'self':
            continue  # Ignore self in constructor

        annotation = signature.parameters[param_name].annotation

        # If the type is a primitive, no extra work needs to be done
        argument = getattr(args, name)
        if annotation in [int, float, str, bool]:
            kwargs[name] = argument
        elif argument is None:
            # Just pass None
            kwargs[name] = None
        else:
            # Otherwise, we are going to try and parse it from a json as if it were parameter
            try:
                params = json.loads(argument)
                kwargs[name] = construct_arg(str(metric_type), name, params, annotation, _NO_DEFAULT)
            except (json.decoder.JSONDecodeError, TypeError):
                # Fall back to just passing the argument
                kwargs[name] = argument

    return metric_type(**kwargs)
